/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonFont.h"

#ifndef CLEARTYPE_QUALITY
	#define CLEARTYPE_QUALITY       5
#endif

NameValuePair alignValues[] = {
		{ "left", DT_LEFT },
		{ "center", DT_CENTER },
		{ "right", DT_RIGHT },
		{ 0, 0 }
		};

NameValuePair vertAlignValues[] = {
		{ "top", DT_TOP },
		{ "center", DT_VCENTER },
		{ "bottom", DT_BOTTOM },
		{ 0, 0 }
		};

NameValuePair CharSetConstants[] =
		{
		{ "ANSI_CHARSET", ANSI_CHARSET },
		{ "BALTIC_CHARSET", BALTIC_CHARSET },
		{ "CHINESEBIG5_CHARSET", CHINESEBIG5_CHARSET },
		{ "DEFAULT_CHARSET", DEFAULT_CHARSET },
		{ "EASTEUROPE_CHARSET", EASTEUROPE_CHARSET },
		{ "GB2312_CHARSET", GB2312_CHARSET },
		{ "GREEK_CHARSET", GREEK_CHARSET },
		{ "HANGUL_CHARSET", HANGUL_CHARSET },
		{ "MAC_CHARSET", MAC_CHARSET },
		{ "RUSSIAN_CHARSET", RUSSIAN_CHARSET },
		{ "SHIFTJIS_CHARSET", SHIFTJIS_CHARSET },
		{ "SYMBOL_CHARSET", SYMBOL_CHARSET },
		{ "TURKISH_CHARSET", TURKISH_CHARSET },
		{ "VIETNAMESE_CHARSET", VIETNAMESE_CHARSET },
		{ "JOHAB_CHARSET", JOHAB_CHARSET },
		{ "ARABIC_CHARSET", ARABIC_CHARSET },
		{ "HEBREW_CHARSET", HEBREW_CHARSET },
		{ "THAI_CHARSET", THAI_CHARSET },
		{ "OEM_CHARSET", OEM_CHARSET },
		{ 0, 0 }
		};

ButtonFont::ButtonFont() :
	scrollPosition(0), //Scrolling
	
	normalFont(0),
	activeFont(0),
	minimizedFont(0),
	flashingFont(0),
	hoverFont(0),
	groupFont(0),
	pressedFont(0),

	tmpFont(0)
{
}

ButtonFont::~ButtonFont()
{
	if(normalFont)
		DeleteObject(normalFont);
	if(activeFont)
		DeleteObject(activeFont);
	if(minimizedFont)
		DeleteObject(minimizedFont);
	if(flashingFont)
		DeleteObject(flashingFont);
	if(hoverFont)
		DeleteObject(hoverFont);
	if(groupFont)
		DeleteObject(groupFont);
	if(pressedFont)
		DeleteObject(pressedFont);

	if(tmpFont)
		DeleteObject(tmpFont);
}

void ButtonFont::subconfigure(string type, FONTSETTINGS &workSettings, FONTSETTINGS &defaultSettings)
{
	workSettings.leftBorder = GetxRCInt(type, "FontLeftBorder", defaultSettings.leftBorder, 0);
	workSettings.rightBorder = GetxRCInt(type, "FontRightBorder", defaultSettings.rightBorder, 0);
	workSettings.topBorder = GetxRCInt(type, "FontTopBorder", defaultSettings.topBorder, 0);
	workSettings.bottomBorder = GetxRCInt(type, "FontBottomBorder", defaultSettings.bottomBorder, 0);

	workSettings.shadow = GetxRCBool(type, "FontShadow", defaultSettings.shadow);
	workSettings.shadowcolor = GetxRCColor(type, "FontShadowColor", defaultSettings.shadowcolor);
	workSettings.shadowX = GetxRCInt(type, "FontShadowX", defaultSettings.shadowX);
	workSettings.shadowY = GetxRCInt(type, "FontShadowY", defaultSettings.shadowY);

	workSettings.textoutline = GetxRCBool(type, "FontOutline", defaultSettings.textoutline);
	workSettings.textoutlinecolor = GetxRCColor(type, "FontOutlineColor", defaultSettings.textoutlinecolor);

	workSettings.name = GetxRCString(type, "Font", defaultSettings.name);
	workSettings.height = GetxRCInt(type, "FontHeight", defaultSettings.height, 0);
	workSettings.color = GetxRCColor(type, "FontColor", defaultSettings.color);

	workSettings.charSet = GetxRCNamedValue(type, "FontCharSet", CharSetConstants, defaultSettings.charSet );

	workSettings.smooth = GetxRCBool(type, "FontSmoothing", defaultSettings.smooth);	
	workSettings.clearType = GetxRCBool(type, "FontClearType", defaultSettings.clearType);

	workSettings.bold = GetxRCBool(type, "FontBold", defaultSettings.bold);
	workSettings.italic = GetxRCBool(type, "FontItalic", defaultSettings.italic);
	workSettings.underline = GetxRCBool(type, "FontUnderline", defaultSettings.underline);

	workSettings.align = GetxRCNamedValue(type, "FontAlign", alignValues, defaultSettings.align);
	workSettings.vertalign = GetxRCNamedValue(type, "FontVertAlign", vertAlignValues, defaultSettings.vertalign);

	workSettings.noEllipsis = GetxRCBool(type, "FontNoEllipsis", defaultSettings.noEllipsis);

	//Initialize Basic Helper Strings
	int numTokens = 0;
	string one = "";
	string two = "";
	string three = "";
	string four = "";
	string five = "";
	string six = "";
	string seven = "";

	workSettings.textFadeCFG = GetxRCLine(type, "FontTextFade", defaultSettings.textFadeCFG);

	numTokens = SplitString(workSettings.textFadeCFG, one, two, three, four, five, six, seven);
		
	if (numTokens == 1)
	{
		workSettings.textFadeLeft = workSettings.textFadeRight = workSettings.textFadeTop = workSettings.textFadeBottom = atoi(one.c_str());
	}
	else if (numTokens == 2)
	{
		workSettings.textFadeLeft = workSettings.textFadeRight = atoi(one.c_str());
		workSettings.textFadeTop = workSettings.textFadeBottom = atoi(two.c_str());
	}
	else if (numTokens == 4)
	{
		workSettings.textFadeLeft = atoi(one.c_str());
		workSettings.textFadeRight = atoi(two.c_str());
		workSettings.textFadeTop = atoi(three.c_str());
		workSettings.textFadeBottom = atoi(four.c_str());
	}

	if ( workSettings.textFadeLeft <= 0 )
		workSettings.textFadeLeft = 0;

	if ( workSettings.textFadeRight <= 0 )
		workSettings.textFadeRight = 0;

	if ( workSettings.textFadeTop <= 0 )
		workSettings.textFadeTop = 0;

	if ( workSettings.textFadeBottom <= 0 )
		workSettings.textFadeBottom = 0;

	if ( IsOS(OS_2KXP) && (workSettings.textFadeLeft > 0 || workSettings.textFadeRight > 0 || workSettings.textFadeTop > 0 || workSettings.textFadeBottom > 0) )
		workSettings.useTextFade = true;
}

void ButtonFont::configure(const string &name)
{
	scrollPadLength = GetxRCInt(name, "FontScrollPad", 10, 0); //Scrolling
	
	defaultfont.leftBorder = GetxRCInt(name, "FontLeftBorder", 0, 0);
	defaultfont.rightBorder = GetxRCInt(name, "FontRightBorder", 0, 0);
	defaultfont.topBorder = GetxRCInt(name, "FontTopBorder", 0, 0);
	defaultfont.bottomBorder = GetxRCInt(name, "FontBottomBorder", 0, 0);

	defaultfont.shadow = GetxRCBool(name, "FontShadow", false);
	defaultfont.shadowcolor = GetxRCColor(name, "FontShadowColor", RGB(128, 128, 128));
	defaultfont.shadowX = GetxRCInt(name, "FontShadowX", 1);
	defaultfont.shadowY = GetxRCInt(name, "FontShadowY", 1);

	defaultfont.textoutline = GetxRCBool(name, "FontOutline", false);
	defaultfont.textoutlinecolor = GetxRCColor(name, "FontOutlineColor", RGB(128, 128, 128));

	defaultfont.name = GetxRCString(name, "Font", "Arial");
	defaultfont.height = GetxRCInt(name, "FontHeight", 15, 0);
	defaultfont.color = GetxRCColor(name, "FontColor", RGB(0, 0, 0));

	defaultfont.charSet = GetxRCNamedValue(name, "FontCharSet", CharSetConstants, DEFAULT_CHARSET );

	defaultfont.smooth = GetxRCBool(name, "FontSmoothing", true);
	defaultfont.clearType = GetxRCBool(name, "FontClearType", true);

	defaultfont.textFadeCFG = GetxRCLine(name, "FontTextFade", "0");
	
	defaultfont.bold = GetxRCBool(name, "FontBold", false);
	defaultfont.italic = GetxRCBool(name, "FontItalic", false);
	defaultfont.underline = GetxRCBool(name, "FontUnderline", false);

	defaultfont.align = GetxRCNamedValue(name, "FontAlign", alignValues, DT_LEFT);
	defaultfont.vertalign = GetxRCNamedValue(name, "FontVertAlign", vertAlignValues, DT_VCENTER);
	defaultfont.noEllipsis = GetxRCBool(name, "FontNoEllipsis");

	specialHover = GetxRCBool(name, "UseHoverFontSettings");
	specialGrouped = GetxRCBool(name, "UseGroupedFontSettings");
	specialPressed = GetxRCBool(name, "UsePressedFontSettings");

	subconfigure((name + "Normal"), normal, defaultfont);
	createHandle(normal, normalFont);

	subconfigure((name + "Active"), active, defaultfont);
	createHandle(active, activeFont);

	subconfigure((name + "Minimized"), minimized, defaultfont);
	createHandle(minimized, minimizedFont);

	subconfigure((name + "Flashing"), flashing, defaultfont);
	createHandle(flashing, flashingFont);

	subconfigure((name + "Hover"), hover, defaultfont);
	createHandle(hover, hoverFont);

	subconfigure((name + "Grouped"), group, defaultfont);
	createHandle(group, groupFont);

	subconfigure((name + "Pressed"), pressed, defaultfont);
	createHandle(pressed, pressedFont);
}

int ButtonFont::measure(HDC hDC, const string &caption, int buttonType)
{
	int neededWidth = 0;

	if (buttonType == 0)
		tmpFont = (HFONT) SelectObject(hDC, normalFont);
	else if (buttonType == 1)
		tmpFont = (HFONT) SelectObject(hDC, activeFont);
	else if (buttonType == 2)
		tmpFont = (HFONT) SelectObject(hDC, minimizedFont);
	else if (buttonType == 3)
		tmpFont = (HFONT) SelectObject(hDC, flashingFont);
	else if (buttonType == 4)
		tmpFont = (HFONT) SelectObject(hDC, hoverFont);
	else if (buttonType == 5)
		tmpFont = (HFONT) SelectObject(hDC, groupFont);
	else if (buttonType == 6)
		tmpFont = (HFONT) SelectObject(hDC, pressedFont);

	LPSIZE lpsize = new SIZE;

	lpsize->cx=300;
	lpsize->cy=100;

	GetTextExtentPoint32(hDC, caption.c_str(), strlen(caption.c_str()), lpsize);

	if (buttonType == 0)
		normalFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 1)
		activeFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 2)
		minimizedFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 3)
		flashingFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 4)
		hoverFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 5)
		groupFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 6)
		pressedFont = (HFONT) SelectObject(hDC, tmpFont);

	neededWidth = lpsize->cx;

    delete lpsize;

	return neededWidth;
}


void ButtonFont::apply(HDC hDC, int x, int y, int width, int height, const string &caption, int buttonType, int buttonType2, bool internalCall)
{
	if ( !internalCall )
	{
		if ( buttonType == 4 && (hover.useTextFade && specialHover) )
		{
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
		else if ( buttonType == 5 && (group.useTextFade && specialGrouped))
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
		else if ( buttonType == 6 && (pressed.useTextFade && specialPressed))
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}

		if ( buttonType == 0 && normal.useTextFade )
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
		else if ( buttonType == 1 && active.useTextFade )
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
		else if ( buttonType == 2 && minimized.useTextFade )
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
		else if ( buttonType == 3 && flashing.useTextFade )
		{	
			applyFade(hDC, x, y, width, height, caption, buttonType, buttonType2);
			return;
		}
	}

	RECT r;

	int textColor;
	int vertAlign;
	int align;

	bool noEllipsis;

	int bkMode = SetBkMode(hDC, TRANSPARENT);

	if ( buttonType == 4 )
	{
		if ( !specialHover )
		{
			buttonType = buttonType2;
		}
		else
		{
			r.left = x + hover.leftBorder;
			r.top = y + hover.topBorder;
			r.right = x + width - hover.rightBorder;
			r.bottom = y + height - hover.bottomBorder;

			tmpFont = (HFONT) SelectObject(hDC, hoverFont);
			textColor = SetTextColor(hDC, hover.color);
			vertAlign = hover.vertalign;
			align = hover.align;

			noEllipsis = hover.noEllipsis;

			if (hover.shadow && !hover.textoutline)
			{
				OffsetRect(&r, (hover.shadowX + 1) / 2, (hover.shadowY + 1) / 2);
				SetTextColor(hDC, hover.shadowcolor);
				DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
				OffsetRect(&r, -hover.shadowX, -hover.shadowY);
				SetTextColor(hDC, hover.color);
			}
			if (hover.textoutline)
			{
				SetTextColor(hDC, hover.textoutlinecolor);
				drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
				SetTextColor(hDC, hover.color);
			}
		}
	}
	else if ( buttonType == 5 )
	{
		if ( !specialGrouped )
		{
			buttonType = buttonType2;
		}
		else
		{
			r.left = x + group.leftBorder;
			r.top = y + group.topBorder;
			r.right = x + width - group.rightBorder;
			r.bottom = y + height - group.bottomBorder;

			tmpFont = (HFONT) SelectObject(hDC, groupFont);
			textColor = SetTextColor(hDC, group.color);
			vertAlign = group.vertalign;
			align = group.align;

			noEllipsis = group.noEllipsis;

			if (group.shadow && !group.textoutline)
			{
				OffsetRect(&r, (group.shadowX + 1) / 2, (group.shadowY + 1) / 2);
				SetTextColor(hDC, group.shadowcolor);
				DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
				OffsetRect(&r, -group.shadowX, -group.shadowY);
				SetTextColor(hDC, group.color);
			}
			if (group.textoutline)
			{
				SetTextColor(hDC, group.textoutlinecolor);
				drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
				SetTextColor(hDC, group.color);
			}
		}
	}
	else if ( buttonType == 6 )
	{
		if ( !specialPressed )
		{
			buttonType = buttonType2;
		}
		else
		{
			r.left = x + pressed.leftBorder;
			r.top = y + pressed.topBorder;
			r.right = x + width - pressed.rightBorder;
			r.bottom = y + height - pressed.bottomBorder;

			tmpFont = (HFONT) SelectObject(hDC, pressedFont);
			textColor = SetTextColor(hDC, pressed.color);
			vertAlign = pressed.vertalign;
			align = pressed.align;

			noEllipsis = pressed.noEllipsis;

			if (pressed.shadow && !pressed.textoutline)
			{
				OffsetRect(&r, (pressed.shadowX + 1) / 2, (pressed.shadowY + 1) / 2);
				SetTextColor(hDC, pressed.shadowcolor);
				DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
				OffsetRect(&r, -pressed.shadowX, -pressed.shadowY);
				SetTextColor(hDC, pressed.color);
			}
			if (pressed.textoutline)
			{
				SetTextColor(hDC, pressed.textoutlinecolor);
				drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
				SetTextColor(hDC, pressed.color);
			}
		}
	}

	if ( buttonType == 0 )
	{
		r.left = x + normal.leftBorder;
		r.top = y + normal.topBorder;
		r.right = x + width - normal.rightBorder;
		r.bottom = y + height - normal.bottomBorder;

		tmpFont = (HFONT) SelectObject(hDC, normalFont);
		textColor = SetTextColor(hDC, normal.color);
		vertAlign = normal.vertalign;
		align = normal.align;

		noEllipsis = normal.noEllipsis;

		if (normal.shadow && !normal.textoutline)
		{
			OffsetRect(&r, (normal.shadowX + 1) / 2, (normal.shadowY + 1) / 2);
			SetTextColor(hDC, normal.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
			OffsetRect(&r, -normal.shadowX, -normal.shadowY);
			SetTextColor(hDC, normal.color);
		}
		if (normal.textoutline)
		{
			SetTextColor(hDC, normal.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
			SetTextColor(hDC, normal.color);
		}
	}
	else if ( buttonType == 1 )
	{
		r.left = x + active.leftBorder;
		r.top = y + active.topBorder;
		r.right = x + width - active.rightBorder;
		r.bottom = y + height - active.bottomBorder;

		tmpFont = (HFONT) SelectObject(hDC, activeFont);
		textColor = SetTextColor(hDC, active.color);
		vertAlign = active.vertalign;
		align = active.align;

		noEllipsis = active.noEllipsis;

		if (active.shadow && !active.textoutline)
		{
			OffsetRect(&r, (active.shadowX + 1) / 2, (active.shadowY + 1) / 2);
			SetTextColor(hDC, active.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
			OffsetRect(&r, -active.shadowX, -active.shadowY);
			SetTextColor(hDC, active.color);
		}
		if (active.textoutline)
		{
			SetTextColor(hDC, active.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
			SetTextColor(hDC, active.color);
		}
	}
	else if ( buttonType == 2 )
	{
		r.left = x + minimized.leftBorder;
		r.top = y + minimized.topBorder;
		r.right = x + width - minimized.rightBorder;
		r.bottom = y + height - minimized.bottomBorder;

		tmpFont = (HFONT) SelectObject(hDC, minimizedFont);
		textColor = SetTextColor(hDC, minimized.color);
		vertAlign = minimized.vertalign;
		align = minimized.align;

		noEllipsis = minimized.noEllipsis;

		if (minimized.shadow && !minimized.textoutline)
		{
			OffsetRect(&r, (minimized.shadowX + 1) / 2, (minimized.shadowY + 1) / 2);
			SetTextColor(hDC, minimized.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
			OffsetRect(&r, -minimized.shadowX, -minimized.shadowY);
			SetTextColor(hDC, minimized.color);
		}
		if (minimized.textoutline)
		{
			SetTextColor(hDC, minimized.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
			SetTextColor(hDC, minimized.color);
		}
	}
	else if ( buttonType == 3 )
	{
		r.left = x + flashing.leftBorder;
		r.top = y + flashing.topBorder;
		r.right = x + width - flashing.rightBorder;
		r.bottom = y + height - flashing.bottomBorder;

		tmpFont = (HFONT) SelectObject(hDC, flashingFont);
		textColor = SetTextColor(hDC, flashing.color);
		vertAlign = flashing.vertalign;
		align = flashing.align;

		noEllipsis = flashing.noEllipsis;

		if (flashing.shadow && !flashing.textoutline)
		{
			OffsetRect(&r, (flashing.shadowX + 1) / 2, (flashing.shadowY + 1) / 2);
			SetTextColor(hDC, flashing.shadowcolor);
			DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
			OffsetRect(&r, -flashing.shadowX, -flashing.shadowY);
			SetTextColor(hDC, flashing.color);
		}
		if (flashing.textoutline)
		{
			SetTextColor(hDC, flashing.textoutlinecolor);
			drawOutline(hDC, r, align, vertAlign, noEllipsis, caption);
			SetTextColor(hDC, flashing.color);
		}
	}

	DrawTextEx(hDC, (char *) caption.c_str(), caption.length(), &r, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	if (buttonType == 0)
		normalFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 1)
		activeFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 2)
		minimizedFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 3)
		flashingFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 4)
		hoverFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 5)
		groupFont = (HFONT) SelectObject(hDC, tmpFont);
	else if (buttonType == 6)
		pressedFont = (HFONT) SelectObject(hDC, tmpFont);

	SetBkMode(hDC, bkMode);
	SetTextColor(hDC, textColor);
}

void ButtonFont::createHandle(FONTSETTINGS processFont, HFONT &hfont)
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, processFont.name.c_str());
	lf.lfHeight = processFont.height;
	lf.lfItalic = processFont.italic ? TRUE : FALSE;
	lf.lfUnderline = processFont.underline ? TRUE : FALSE;
	lf.lfWeight = processFont.bold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = processFont.charSet;
	if ( IsOS( OS_XP ) && processFont.smooth && processFont.clearType )
		lf.lfQuality = CLEARTYPE_QUALITY;
	else if ( !processFont.smooth )
		lf.lfQuality = NONANTIALIASED_QUALITY;

	if(hfont != 0)
		DeleteObject(hfont);

	hfont = CreateFontIndirect(&lf);
}

void ButtonFont::drawOutline(HDC destHDC, RECT tmpr, int align, int vertAlign, bool noEllipsis, const string &caption)
{
	OffsetRect(&tmpr, 1, 1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, -1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, -1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, 0, -1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, 0, -1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, 1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, 1, 0);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);

	OffsetRect(&tmpr, 0, 1);
	DrawTextEx(destHDC, (char *) caption.c_str(), caption.length(), &tmpr, align | DT_SINGLELINE | DT_NOPREFIX | vertAlign | (noEllipsis ? 0: DT_END_ELLIPSIS), 0);
}

void ButtonFont::applyFade(HDC hDC, int x, int y, int width, int height, const string &caption, int buttonType, int buttonType2)
{
	if ( width <= 0 || height <= 0 )
		return;

	HDC textFadeDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP textFadeBitmap = CreateDIBSection( textFadeDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(textFadeDC, textFadeBitmap);

	BitBlt(textFadeDC, 0, 0, width, height, hDC, x, y, SRCCOPY);

	BITMAP bm;
	GetObject(textFadeBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		pPixel[3] = 1;

		pPixel+= 4;
	}
	
	if ( buttonType == 4 && !specialHover )
		buttonType = buttonType2;
	else if ( buttonType == 5 && !specialGrouped )
		buttonType = buttonType2;
	else if ( buttonType == 6 && !specialPressed )
		buttonType = buttonType2;

	apply(textFadeDC, 0, 0, width, height, caption, buttonType, buttonType2, true/*internalCall*/);

	GetObject(textFadeBitmap, sizeof(BITMAP), &bm);
	pPixel = (LPBYTE)bm.bmBits;

	int textFadeLeft = 0;
	int textFadeRight = 0;
	int textFadeTop = 0;
	int textFadeBottom = 0;

	int leftBorder = 0;
	int rightBorder = 0;
	int topBorder = 0;
	int bottomBorder = 0;

	if (buttonType == 0)
	{
		textFadeRight = normal.textFadeRight;
		textFadeLeft = normal.textFadeLeft;
		textFadeTop = normal.textFadeTop;
		textFadeBottom = normal.textFadeBottom;

		leftBorder = normal.leftBorder;
		rightBorder = normal.rightBorder;
		topBorder = normal.topBorder;
		bottomBorder = normal.bottomBorder;
	}
	else if (buttonType == 1)
	{
		textFadeRight = active.textFadeRight;
		textFadeLeft = active.textFadeLeft;
		textFadeTop = active.textFadeTop;
		textFadeBottom = active.textFadeBottom;

		leftBorder = active.leftBorder;
		rightBorder = active.rightBorder;
		topBorder = active.topBorder;
		bottomBorder = active.bottomBorder;
	}
	else if (buttonType == 2)
	{
		textFadeRight = minimized.textFadeRight;
		textFadeLeft = minimized.textFadeLeft;
		textFadeTop = minimized.textFadeTop;
		textFadeBottom = minimized.textFadeBottom;

		leftBorder = minimized.leftBorder;
		rightBorder = minimized.rightBorder;
		topBorder = minimized.topBorder;
		bottomBorder = minimized.bottomBorder;
	}
	else if (buttonType == 3)
	{
		textFadeRight = flashing.textFadeRight;
		textFadeLeft = flashing.textFadeLeft;
		textFadeTop = flashing.textFadeTop;
		textFadeBottom = flashing.textFadeBottom;

		leftBorder = flashing.leftBorder;
		rightBorder = flashing.rightBorder;
		topBorder = flashing.topBorder;
		bottomBorder = flashing.bottomBorder;
	}
	else if (buttonType == 4)
	{
		textFadeRight = hover.textFadeRight;
		textFadeLeft = hover.textFadeLeft;
		textFadeTop = hover.textFadeTop;
		textFadeBottom = hover.textFadeBottom;

		leftBorder = hover.leftBorder;
		rightBorder = hover.rightBorder;
		topBorder = hover.topBorder;
		bottomBorder = hover.bottomBorder;
	}
	else if (buttonType == 5)
	{
		textFadeRight = group.textFadeRight;
		textFadeLeft = group.textFadeLeft;
		textFadeTop = group.textFadeTop;
		textFadeBottom = group.textFadeBottom;

		leftBorder = group.leftBorder;
		rightBorder = group.rightBorder;
		topBorder = group.topBorder;
		bottomBorder = group.bottomBorder;
	}
	else if (buttonType == 6)
	{
		textFadeRight = pressed.textFadeRight;
		textFadeLeft = pressed.textFadeLeft;
		textFadeTop = pressed.textFadeTop;
		textFadeBottom = pressed.textFadeBottom;

		leftBorder = pressed.leftBorder;
		rightBorder = pressed.rightBorder;
		topBorder = pressed.topBorder;
		bottomBorder = pressed.bottomBorder;
	}

	int tmpx = 0;
	int tmpy = 0;
	int xval = 255;
	int yval = 255;

	iSize = bm.bmHeight * bm.bmWidth;

	for (j = 0; j < iSize; ++j)
	{
		//Convert Text to Full Opacity
		if (pPixel[3] == 0)
		{
			xval = 255;
			yval = 255;

			tmpx = (j+1)%width;
			tmpy = j/width;

			if ( tmpx == 0 )
				tmpx = width-1;

			if ( textFadeLeft != 0 && tmpx > leftBorder && tmpx <= leftBorder+textFadeLeft )
			{
				xval = (255/textFadeLeft)*(tmpx-leftBorder);
			}
			else if ( textFadeRight != 0 && tmpx > width-rightBorder-textFadeRight && tmpx <= width-rightBorder )
			{
				xval = (255/textFadeRight)*(width-rightBorder-tmpx);
			}
			
			//Order is reverted Top<->Bottom
			if ( textFadeBottom != 0 && tmpy > bottomBorder && tmpy < bottomBorder+textFadeBottom )
			{
				yval = (255/textFadeBottom)*(tmpy-bottomBorder);
			}
			else if ( textFadeTop != 0 && tmpy >= height-topBorder-textFadeTop && tmpy < height-topBorder )
			{
				yval = (255/textFadeTop)*(height-topBorder-tmpy);
			}
			
			if ( xval == 255 )
				pPixel[3] = yval;
			else if ( yval == 255 )
				pPixel[3] = xval;
			else
				pPixel[3] = (xval*yval)/255;
				
			//PreMultiply
			pPixel[0] = pPixel[0] * pPixel[3] / 255;
			pPixel[1] = pPixel[1] * pPixel[3] / 255;
			pPixel[2] = pPixel[2] * pPixel[3] / 255;
		}
		else
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0; 
		}
					
		pPixel+= 4;	
	}

	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, textFadeDC, 0, 0, width, height, blendFunction);

	textFadeBitmap = (HBITMAP) SelectObject(textFadeDC, oldBMP);
	DeleteObject(textFadeBitmap);
	DeleteDC(textFadeDC);
}

void ButtonFont::applyScroll(HDC hDC, int x, int y, int width, int height, const string &caption, int buttonType, int buttonType2)
{
	int textFadeLeft = 0;
	int textFadeRight = 0;
	int textFadeTop = 0;
	int textFadeBottom = 0;

	int leftBorder = 0;
	int rightBorder = 0;
	int topBorder = 0;
	int bottomBorder = 0;

	bool useTextFade = false;

	if ( buttonType == 4 && !specialHover )
		buttonType = buttonType2;
	else if ( buttonType == 5 && !specialGrouped )
		buttonType = buttonType2;
	else if ( buttonType == 6 && !specialPressed )
		buttonType = buttonType2;

	if (buttonType == 0)
	{
		useTextFade = normal.useTextFade;

		textFadeRight = normal.textFadeRight;
		textFadeLeft = normal.textFadeLeft;
		textFadeTop = normal.textFadeTop;
		textFadeBottom = normal.textFadeBottom;

		leftBorder = normal.leftBorder;
		rightBorder = normal.rightBorder;
		topBorder = normal.topBorder;
		bottomBorder = normal.bottomBorder;
	}
	else if (buttonType == 1)
	{
		useTextFade = active.useTextFade;

		textFadeRight = active.textFadeRight;
		textFadeLeft = active.textFadeLeft;
		textFadeTop = active.textFadeTop;
		textFadeBottom = active.textFadeBottom;

		leftBorder = active.leftBorder;
		rightBorder = active.rightBorder;
		topBorder = active.topBorder;
		bottomBorder = active.bottomBorder;
	}
	else if (buttonType == 2)
	{
		useTextFade = minimized.useTextFade;

		textFadeRight = minimized.textFadeRight;
		textFadeLeft = minimized.textFadeLeft;
		textFadeTop = minimized.textFadeTop;
		textFadeBottom = minimized.textFadeBottom;

		leftBorder = minimized.leftBorder;
		rightBorder = minimized.rightBorder;
		topBorder = minimized.topBorder;
		bottomBorder = minimized.bottomBorder;
	}
	else if (buttonType == 3)
	{
		useTextFade = flashing.useTextFade;

		textFadeRight = flashing.textFadeRight;
		textFadeLeft = flashing.textFadeLeft;
		textFadeTop = flashing.textFadeTop;
		textFadeBottom = flashing.textFadeBottom;

		leftBorder = flashing.leftBorder;
		rightBorder = flashing.rightBorder;
		topBorder = flashing.topBorder;
		bottomBorder = flashing.bottomBorder;
	}
	else if (buttonType == 4)
	{
		useTextFade = hover.useTextFade;

		textFadeRight = hover.textFadeRight;
		textFadeLeft = hover.textFadeLeft;
		textFadeTop = hover.textFadeTop;
		textFadeBottom = hover.textFadeBottom;

		leftBorder = hover.leftBorder;
		rightBorder = hover.rightBorder;
		topBorder = hover.topBorder;
		bottomBorder = hover.bottomBorder;
	}
	else if (buttonType == 5)
	{
		useTextFade = group.useTextFade;

		textFadeRight = group.textFadeRight;
		textFadeLeft = group.textFadeLeft;
		textFadeTop = group.textFadeTop;
		textFadeBottom = group.textFadeBottom;

		leftBorder = group.leftBorder;
		rightBorder = group.rightBorder;
		topBorder = group.topBorder;
		bottomBorder = group.bottomBorder;
	}
	else if (buttonType == 6)
	{
		useTextFade = pressed.useTextFade;

		textFadeRight = pressed.textFadeRight;
		textFadeLeft = pressed.textFadeLeft;
		textFadeTop = pressed.textFadeTop;
		textFadeBottom = pressed.textFadeBottom;

		leftBorder = pressed.leftBorder;
		rightBorder = pressed.rightBorder;
		topBorder = pressed.topBorder;
		bottomBorder = pressed.bottomBorder;
	}
	//--------------------------------------------------------------------------------
	// start rendering text
	//--------------------------------------------------------------------------------
	
	int textWidth = measure(hDC, caption, buttonType);
	//Create Region in which text drawing is allowed
	HRGN allowedArea = CreateRectRgn( x+leftBorder, y+topBorder, x+width-rightBorder, y+height-bottomBorder );
	SelectClipRgn(hDC, allowedArea);
	DeleteObject(allowedArea);
	
	if ( textWidth >= width-leftBorder-rightBorder )
	{			
		if(scrollPosition <= 0)
			scrollPosition = textWidth + scrollPadLength;

		if(scrollPosition > textWidth + scrollPadLength)
			scrollPosition = 0;
		
		if ( useTextFade && IsOS( OS_2KXP ) )
		{
			HDC textFadeDC = CreateCompatibleDC(NULL);

			BITMAPINFO alpha;
			ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
			alpha.bmiHeader.biWidth=width;      // Set size you need
			alpha.bmiHeader.biHeight=height;    // Set size you need
			alpha.bmiHeader.biPlanes=1;
			alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
			alpha.bmiHeader.biSizeImage=0;
			alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
			alpha.bmiHeader.biClrUsed= 0;
			alpha.bmiHeader.biClrImportant= 0;
			VOID *pvBits;

			HBITMAP textFadeBitmap = CreateDIBSection( textFadeDC,
														&alpha,
														DIB_RGB_COLORS,
														&pvBits,
														NULL,
														0 );

			HBITMAP oldBMP = (HBITMAP) SelectObject(textFadeDC, textFadeBitmap);

			BitBlt(textFadeDC, 0, 0, width, height, hDC, x, y, SRCCOPY);

			BITMAP bm;
			GetObject(textFadeBitmap, sizeof(BITMAP), &bm);

			BYTE *pPixel = (LPBYTE)bm.bmBits;

			int iSize = bm.bmHeight * bm.bmWidth;

			for (int j = 0; j < iSize; ++j)
			{
				pPixel[3] = 1;

				pPixel+= 4;
			}

			apply(textFadeDC,
					scrollPosition,
					0,
					2*textWidth,
					height,
					caption,
					buttonType,
					buttonType2, true/*internalCall*/);

			apply(textFadeDC,
					scrollPosition - scrollPadLength - textWidth,
					0,
					2*textWidth,
					height,
					caption,
					buttonType,
					buttonType2, true/*internalCall*/);

			GetObject(textFadeBitmap, sizeof(BITMAP), &bm);
			pPixel = (LPBYTE)bm.bmBits;

			int tmpx = 0;
			int tmpy = 0;
			int xval = 255;
			int yval = 255;

			iSize = bm.bmHeight * bm.bmWidth;

			for (j = 0; j < iSize; ++j)
			{
				//Convert Text to Full Opacity
				if (pPixel[3] == 0)
				{
					xval = 255;
					yval = 255;

					tmpx = (j+1)%width;
					tmpy = j/width;

					if ( tmpx == 0 )
						tmpx = width-1;

					if ( textFadeLeft != 0 && tmpx > leftBorder && tmpx <= leftBorder+textFadeLeft )
					{
						xval = (255/textFadeLeft)*(tmpx-leftBorder);
					}
					else if ( textFadeRight != 0 && tmpx > width-rightBorder-textFadeRight && tmpx <= width-rightBorder )
					{
						xval = (255/textFadeRight)*(width-rightBorder-tmpx);
					}
					
					//Order is reverted Top<->Bottom
					if ( textFadeBottom != 0 && tmpy > bottomBorder && tmpy < bottomBorder+textFadeBottom )
					{
						yval = (255/textFadeBottom)*(tmpy-bottomBorder);
					}
					else if ( textFadeTop != 0 && tmpy >= height-topBorder-textFadeTop && tmpy < height-topBorder )
					{
						yval = (255/textFadeTop)*(height-topBorder-tmpy);
					}
					
					if ( xval == 255 )
						pPixel[3] = yval;
					else if ( yval == 255 )
						pPixel[3] = xval;
					else
						pPixel[3] = (xval*yval)/255;
						
					//PreMultiply
					pPixel[0] = pPixel[0] * pPixel[3] / 255;
					pPixel[1] = pPixel[1] * pPixel[3] / 255;
					pPixel[2] = pPixel[2] * pPixel[3] / 255;
				}
				else
				{
					pPixel[0] = 0;
					pPixel[1] = 0;
					pPixel[2] = 0;
					pPixel[3] = 0; 
				}
							
				pPixel+= 4;	
			}

			BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, AC_SRC_ALPHA };

			AlphaBlend(hDC, x, y, width, height, textFadeDC, 0, 0, width, height, blendFunction);

			textFadeBitmap = (HBITMAP) SelectObject(textFadeDC, oldBMP);
			DeleteObject(textFadeBitmap);
			DeleteDC(textFadeDC);
		}
		else
		{
			apply(hDC,
					x + scrollPosition,
					y,
					2*textWidth,
					height,
					caption,
					buttonType,
					buttonType2, true/*internalCall*/);

			apply(hDC,
					x + scrollPosition - scrollPadLength - textWidth,
					y,
					2*textWidth,
					height,
					caption,
					buttonType,
					buttonType2, true/*internalCall*/);
		}
	}
	else
	{
		apply(hDC, x, y, width, height, caption, buttonType, buttonType2);
	}

	//Reset Region in which text drawing is allowed
	SelectClipRgn(hDC, NULL);
	//--------------------------------------------------------------------------------
	// stop rendering text
	//--------------------------------------------------------------------------------
}