/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonIcon.h"

ButtonIcon::ButtonIcon()
{
}

ButtonIcon::~ButtonIcon()
{
}

void ButtonIcon::subconfigure(string type, ICONSETTINGS &workSettings, ICONSETTINGS &defaultSettings)
{
	workSettings.iconSize = GetxRCInt(type, "IconSize", defaultSettings.iconSize, 8, 64);
	workSettings.iconX = GetxRCString(type, "IconX", defaultSettings.iconX);
	if ( defaulticon.iconY.empty() )
		workSettings.iconY = GetxRCString(type, "IconY", ("-" + IntToString(workSettings.iconSize/2) + "c"));
	else
		workSettings.iconY = GetxRCString(type, "IconY", defaulticon.iconY);

	workSettings.icon_hueColor = GetxRCColor(type, "IconHueColor", defaultSettings.icon_hueColor);
	workSettings.icon_hueIntensity = GetxRCInt(type, "IconHueIntensity", defaultSettings.icon_hueIntensity, 0, 100);
	workSettings.icon_luminanceIntensity = GetxRCInt(type, "IconLuminanceIntensity", defaultSettings.icon_luminanceIntensity, -100, 100);
	workSettings.icon_saturationIntensity = GetxRCInt(type, "IconSaturationIntensity", defaultSettings.icon_saturationIntensity, 0, 200);

	workSettings.icon_mixColor = GetxRCColor(type, "IconMixColor", defaultSettings.icon_mixColor);
	workSettings.icon_mixIntensity = GetxRCInt(type, "IconMixIntensity", defaultSettings.icon_mixIntensity, 0, 100);
}

void ButtonIcon::configure(const string &name)
{
	defaulticon.iconSize = GetxRCInt(name, "IconSize", 16, 8, 64);
	defaulticon.iconX = GetxRCString(name, "IconX", "0");
	defaulticon.iconY = GetxRCString(name, "IconY", "" );
	
	defaulticon.icon_hueColor = GetxRCColor(name, "IconHueColor", RGB(255, 255, 255));
	defaulticon.icon_hueIntensity = GetxRCInt(name, "IconHueIntensity", 0, 0, 100);
	defaulticon.icon_luminanceIntensity = GetxRCInt(name, "IconLuminanceIntensity", 0, -100, 100);
	defaulticon.icon_saturationIntensity = GetxRCInt(name, "IconSaturationIntensity", 100, 0, 200);

	defaulticon.icon_mixColor = GetxRCColor(name, "IconMixColor", RGB(255, 255, 255));
	defaulticon.icon_mixIntensity = GetxRCInt(name, "IconMixIntensity", 0, 0, 100);

	specialHover = GetxRCBool(name, "UseHoverIconSettings");
	specialGrouped = GetxRCBool(name, "UseGroupedIconSettings");
	specialPressed = GetxRCBool(name, "UsePressedIconSettings");

	subconfigure((name + "Normal"), normal, defaulticon);

	subconfigure((name + "Active"), active, defaulticon);

	subconfigure((name + "Minimized"), minimized, defaulticon);

	subconfigure((name + "Flashing"), flashing, defaulticon);

	subconfigure((name + "Hover"), hover, defaulticon);

	subconfigure((name + "Grouped"), group, defaulticon);

	subconfigure((name + "Pressed"), pressed, defaulticon);
}

void ButtonIcon::apply(HDC hDC, int x, int y, int width, int height, int buttonType, int buttonType2, HICON hIcon, bool useAlphaMap)
{
	if(hIcon == NULL)
		return;

	int iconX = 0;
	int iconY = 0;
	int _iconSize = 16;

	COLORREF _hueColor = RGB(255, 255, 255);
	int _hueIntensity = 0;
	int _luminanceIntensity = 0;
	int _saturationIntensity = 100;

	COLORREF _mixColor = RGB(255, 255, 255);
	int _mixIntensity = 0;

	if (buttonType == 4)
	{
		if ( !specialHover )
		{
			buttonType = buttonType2;
		}
		else
		{
			iconX = xParseCoord(hover.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
			iconY = xParseCoord(hover.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

			_iconSize = hover.iconSize;
			
			_hueColor = hover.icon_hueColor;
			_hueIntensity = hover.icon_hueIntensity;
			_luminanceIntensity = hover.icon_luminanceIntensity;
			_saturationIntensity = hover.icon_saturationIntensity;

			_mixColor = hover.icon_mixColor;
			_mixIntensity = hover.icon_mixIntensity;
		}
	}
	else if (buttonType == 5)
	{
		if ( !specialGrouped )
		{
			buttonType = buttonType2;
		}
		else
		{
			iconX = xParseCoord(group.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
			iconY = xParseCoord(group.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

			_iconSize = group.iconSize;
		
			_hueColor = group.icon_hueColor;
			_hueIntensity = group.icon_hueIntensity;
			_luminanceIntensity = group.icon_luminanceIntensity;
			_saturationIntensity = group.icon_saturationIntensity;

			_mixColor = group.icon_mixColor;
			_mixIntensity = group.icon_mixIntensity;
		}
	}
	else if (buttonType == 6)
	{
		if ( !specialPressed )
		{
			buttonType = buttonType2;
		}
		else
		{
			iconX = xParseCoord(pressed.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
			iconY = xParseCoord(pressed.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

			_iconSize = pressed.iconSize;
		
			_hueColor = pressed.icon_hueColor;
			_hueIntensity = pressed.icon_hueIntensity;
			_luminanceIntensity = pressed.icon_luminanceIntensity;
			_saturationIntensity = pressed.icon_saturationIntensity;

			_mixColor = pressed.icon_mixColor;
			_mixIntensity = pressed.icon_mixIntensity;
		}
	}

	if (buttonType == 0)
	{
		iconX = xParseCoord(normal.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(normal.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = normal.iconSize;
	
		_hueColor = normal.icon_hueColor;
		_hueIntensity = normal.icon_hueIntensity;
		_luminanceIntensity = normal.icon_luminanceIntensity;
		_saturationIntensity = normal.icon_saturationIntensity;

		_mixColor = normal.icon_mixColor;
		_mixIntensity = normal.icon_mixIntensity;
	}
	else if (buttonType == 1)
	{
		iconX = xParseCoord(active.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(active.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = active.iconSize;
	
		_hueColor = active.icon_hueColor;
		_hueIntensity = active.icon_hueIntensity;
		_luminanceIntensity = active.icon_luminanceIntensity;
		_saturationIntensity = active.icon_saturationIntensity;

		_mixColor = active.icon_mixColor;
		_mixIntensity = active.icon_mixIntensity;
	}
	else if (buttonType == 2)
	{
		iconX = xParseCoord(minimized.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(minimized.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = minimized.iconSize;
	
		_hueColor = minimized.icon_hueColor;
		_hueIntensity = minimized.icon_hueIntensity;
		_luminanceIntensity = minimized.icon_luminanceIntensity;
		_saturationIntensity = minimized.icon_saturationIntensity;

		_mixColor = minimized.icon_mixColor;
		_mixIntensity = minimized.icon_mixIntensity;
	}
	else if (buttonType == 3)
	{
		iconX = xParseCoord(flashing.iconX, 0, width, GetLitestepDesktop()); //GetLitestepDesktop as placeholder
		iconY = xParseCoord(flashing.iconY, 0, height, GetLitestepDesktop()); //GetLitestepDesktop as placeholder

		_iconSize = flashing.iconSize;
	
		_hueColor = flashing.icon_hueColor;
		_hueIntensity = flashing.icon_hueIntensity;
		_luminanceIntensity = flashing.icon_luminanceIntensity;
		_saturationIntensity = flashing.icon_saturationIntensity;

		_mixColor = flashing.icon_mixColor;
		_mixIntensity = flashing.icon_mixIntensity;
	}

	if ( useAlphaMap || _hueIntensity != 0 || _luminanceIntensity != 0 || _saturationIntensity != 100 || _mixIntensity != 0 )
	{	
		BITMAPINFO alpha;
		ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
		alpha.bmiHeader.biWidth=_iconSize;      // Set size you need
		alpha.bmiHeader.biHeight=_iconSize;    // Set size you need
		alpha.bmiHeader.biPlanes=1;
		alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
		alpha.bmiHeader.biSizeImage=0;
		alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
		alpha.bmiHeader.biClrUsed= 0;
		alpha.bmiHeader.biClrImportant= 0;
		VOID *pvBits;
		
		HDC BGDC = CreateCompatibleDC(NULL);
		HBITMAP	BGBitmap = CreateDIBSection( BGDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );
		HBITMAP oldBGBitmap = (HBITMAP) SelectObject(BGDC, BGBitmap);

		//Copy RealBG in Result Background
		BitBlt( BGDC, 0, 0, _iconSize, _iconSize, hDC, x+iconX, y+iconY, SRCCOPY );
	
		HDC bufferDC = CreateCompatibleDC(NULL);
		HBITMAP	bufferBitmap = CreateDIBSection( bufferDC,
													&alpha,
													DIB_RGB_COLORS,
													&pvBits,
													NULL,
													0 );

		HBITMAP oldbufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		//Copy RealBG in Buffer Background
		BitBlt( bufferDC, 0, 0, _iconSize, _iconSize, hDC, x+iconX, y+iconY, SRCCOPY );
		
		DrawIconEx(bufferDC, 0, 0, hIcon, _iconSize, _iconSize, 0, NULL, DI_NORMAL);

		HDC maskDC = CreateCompatibleDC(bufferDC);
		HBITMAP maskBitmap = CreateDIBSection( maskDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

		HBITMAP oldmaskBitmap = (HBITMAP) SelectObject(maskDC, maskBitmap);

		DrawIconEx(maskDC, 0, 0, hIcon, _iconSize, _iconSize, 0, NULL, DI_MASK);

		if ( IsOS( OS_XP ) )
		{
			BITMAP bm_b;
			GetObject(bufferBitmap, sizeof(BITMAP), &bm_b);
			BYTE *pPixel_b = (LPBYTE)bm_b.bmBits;

			BITMAP bm_m;
			GetObject(maskBitmap, sizeof(BITMAP), &bm_m);
			BYTE *pPixel_m = (LPBYTE)bm_m.bmBits;

			BITMAP bm_bg;
			GetObject(BGBitmap, sizeof(BITMAP), &bm_bg);
			BYTE *pPixel_bg = (LPBYTE)bm_bg.bmBits;

			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;

			BYTE h = 0;
			BYTE s = 0;
			BYTE l = 0;
			
			BYTE hueh = 0;
			BYTE hues = 0;
			BYTE huel = 0;

			bool needHSLConversion = false;
			bool needMixColor = false;

			if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
				needHSLConversion = true;

			if ( _hueIntensity != 0 )
			{
				r = GetRValue(_hueColor);
				g = GetGValue(_hueColor);
				b = GetBValue(_hueColor);

				if ( r == b && b == g )
				{
					_hueIntensity = 0;
				}
				else
				{
					RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
					needHSLConversion = true;
				}
			}

			if ( _mixIntensity != 0 )
			{
				_mixIntensity = (int) min(255, _mixIntensity*2.55);
				needMixColor = true;
			}

			bool skip = false;

			int iSize = bm_b.bmHeight * bm_b.bmWidth;
		
			for ( int j = 0; j < iSize; ++j )
			{		
				//Filter Mask and insert OriginalBG to preserve AlphaChannel
				if ( pPixel_m[0] == 255 && pPixel_m[1] == 255 && pPixel_m[2] == 255 )
				{
					pPixel_b[0] = pPixel_bg[0];	
					pPixel_b[1] = pPixel_bg[1];	
					pPixel_b[2] = pPixel_bg[2];
					pPixel_b[3] = pPixel_bg[3];	

					skip = true;
				}
				//For colorizing without AlphaMap
				if ( !skip && pPixel_bg[0] == 255 && pPixel_bg[1] == 0 && pPixel_bg[2] == 255 && pPixel_b[3] != 255 && pPixel_b[3] != 0 )
				{
					pPixel_b[0] = pPixel_bg[0];	
					pPixel_b[1] = pPixel_bg[1];	
					pPixel_b[2] = pPixel_bg[2];	
					pPixel_b[3] = pPixel_bg[3];	

					skip = true;
				}

				if ( skip )
				{
					pPixel_b+= 4;
					pPixel_m+= 4;
					pPixel_bg+= 4;

					skip = false;
					continue;
				}

				r = pPixel_b[0];
				g = pPixel_b[1];
				b = pPixel_b[2];

				if ( needHSLConversion )
				{
					//AlphaMap Colorizing
					RGB2HSL(r, g, b, h, s, l);

					if(_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
				
					if(_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

					if(_saturationIntensity != 100)
					{
						if ( _saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
					}

					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix Effect
					if ( pPixel_b[3] != 0 )
					{
						r = (BYTE)((r * (255 - _mixIntensity) + (pPixel_b[3]*GetBValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _mixIntensity) + (pPixel_b[3]*GetGValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _mixIntensity) + (pPixel_b[3]*GetRValue(_mixColor)/255) * _mixIntensity + 255) >> 8);
					}
					else
					{
						r = (BYTE)((r * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
						g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
						b = (BYTE)((b * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
					}
				}

				pPixel_b[0] = r;
				pPixel_b[1] = g;
				pPixel_b[2] = b;

				//Fix for Icons without AlphaChannel (all is 0!), with AlphaChannel 0 is Mask
				if ( pPixel_b[3] == 0 )
					pPixel_b[3] = 255;

				pPixel_b+= 4;
				pPixel_m+= 4;
				pPixel_bg+= 4;
			}

			BitBlt( hDC, x+iconX, y+iconY, _iconSize, _iconSize, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			int iy = 0;
			int ix = 0;
			COLORREF cl = 0;
			BYTE r = 0;
			BYTE g = 0;
			BYTE b = 0;

			BYTE h = 0;
			BYTE s = 0;
			BYTE l = 0;
			
			BYTE hueh = 0;
			BYTE hues = 0;
			BYTE huel = 0;

			bool needHSLConversion = false;
			bool needMixColor = false;

			if ( _luminanceIntensity != 0 || _saturationIntensity != 100 )
				needHSLConversion = true;

			if ( _hueIntensity != 0 )
			{
				r = GetRValue(_hueColor);
				g = GetGValue(_hueColor);
				b = GetBValue(_hueColor);

				if ( r == b && b == g )
				{
					_hueIntensity = 0;
				}
				else
				{
					RGB2HSL(r, g, b, hueh, hues, huel);
					needHSLConversion = true;
				}
			}

			if ( _mixIntensity != 0 )
			{
				_mixIntensity = (int) min(255, _mixIntensity*2.55);
				needMixColor = true;
			}

			for(iy = 0; iy < _iconSize; iy++)
			{
				for(ix = 0; ix < _iconSize; ix++)
				{
					// Loop through all transparent pixels...
					// If the mask-pixel is white or grey(if that is possible?), then break
					if( !GetPixel(maskDC, ix, iy) )
					{
						cl = GetPixel(bufferDC, ix, iy);
						r = GetRValue(cl);
						g = GetGValue(cl);
						b = GetBValue(cl);

						if ( needHSLConversion )
						{
							RGB2HSL(r, g, b, h, s, l);

							if(_hueIntensity != 0)
								h = (BYTE) max(0, min(240, (h*(100-_hueIntensity)+hueh*_hueIntensity)/100));
				
							if(_luminanceIntensity != 0)
								l = (BYTE) max(0, min(240, ((l*100)+(120*_luminanceIntensity))/100));

							if(_saturationIntensity != 100)
							{
								if ( _saturationIntensity > 100 )
									s = (BYTE) max(0, min(240, (240*(_saturationIntensity-100))/100));				
								else
									s = (BYTE) max(0, min(240, (s*_saturationIntensity)/100));
							}
						
							HSL2RGB(r, g, b, h, s, l);
						}

						if ( needMixColor )
						{
							// HueMix Effect
							r = (BYTE)((r * (255 - _mixIntensity) + GetRValue(_mixColor) * _mixIntensity + 255) >> 8);
							g = (BYTE)((g * (255 - _mixIntensity) + GetGValue(_mixColor) * _mixIntensity + 255) >> 8);
							b = (BYTE)((b * (255 - _mixIntensity) + GetBValue(_mixColor) * _mixIntensity + 255) >> 8);
						}

						SetPixelV(BGDC, ix, iy, RGB(r,g,b));
					}
				}
			}

			BitBlt( hDC, x+iconX, y+iconY, _iconSize, _iconSize, BGDC, 0, 0, SRCCOPY);
		}


		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldbufferBitmap);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);

		maskBitmap = (HBITMAP) SelectObject(maskDC, oldmaskBitmap);
		DeleteObject(maskBitmap);
		DeleteDC(maskDC);

		BGBitmap = (HBITMAP) SelectObject(BGDC, oldBGBitmap);
		DeleteObject(BGBitmap);
		DeleteDC(BGDC);
	}
	else
	{
		DrawIconEx(hDC, x+iconX, y+iconY, hIcon, _iconSize, _iconSize, 0, NULL, DI_NORMAL);
	}
}

