/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonTexture.h"
#include "math.h"

NameValuePair modeConstants[] =
{
	{ "stretch", MULTIBLT_STRETCH },
	{ "tile", MULTIBLT_TILE },
	{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
	{ "tile-vertical", MULTIBLT_TILEVERTICAL },
	{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
	{ "tilevertical", MULTIBLT_TILEVERTICAL },
	{ 0, 0 }
};

NameValuePair borderValues[] = 
{
	{ ".none", 0 },
	{ "none", 0 },
	{ "raised", 1 },
	{ "sunken", 2 },
	{ "etched", 3 },
	{ "bump", 4 },
	{ 0, 0 }
};

NameValuePair gradientmodeConstants[] =
{
	{ ".none", GRADIENT_NONE },
	{ ".horizontal", GRADIENT_HORIZONTAL },
	{ ".vertical", GRADIENT_VERTICAL },
	{ ".radial", GRADIENT_RADIAL },
	{ ".diagonal", GRADIENT_DIAGONAL },
	{ ".fdiagonal", GRADIENT_FDIAGONAL },
	{ ".bdiagonal", GRADIENT_BDIAGONAL },
	{ 0, 0 }
};

NameValuePair gradienttransformationConstants[] =
{
	{ ".none", TRANSFORMATION_NONE },
	{ ".caricature", TRANSFORMATION_CHARICATURE },
	{ ".fisheye", TRANSFORMATION_FISHEYE },
	{ ".swirled", TRANSFORMATION_SWIRLED },
	{ ".cylinder", TRANSFORMATION_CYLINDER },
	{ ".shift", TRANSFORMATION_SHIFT },
	{ 0, 0 }
};

#define COLOR_GRADIENTACTIVECAPTION     27
#define COLOR_GRADIENTINACTIVECAPTION   28

ButtonTexture::ButtonTexture() :
	m_Data(NULL),

	transparencyNeeded(false),

	useAlphaMap(false),

	memoryNormalDC(0),
	memoryNormalBitmap(0),

	memoryActiveDC(0),
	memoryActiveBitmap(0),

	memoryMinimizedDC(0),
	memoryMinimizedBitmap(0),

	memoryFlashingDC(0),
	memoryFlashingBitmap(0), 

	memoryHoverDC(0),
	memoryHoverBitmap(0), 

	memoryGroupDC(0),
	memoryGroupBitmap(0), 

	memoryPressedDC(0),
	memoryPressedBitmap(0),

	usenormalSolidColors(false),
	useactiveSolidColors(false),
	useminimizedSolidColors(false),
	useflashingSolidColors(false),

	usehoverSolidColors(false),
	usegroupSolidColors(false),
	usepressedSolidColors(false)
{
}

ButtonTexture::~ButtonTexture()
{
	if ( m_Data != NULL )
		delete m_Data;

	NormalGradient.clear();
	ActiveGradient.clear();
	MinimizedGradient.clear();
	FlashingGradient.clear();

	HoverGradient.clear();
	GroupGradient.clear();
	PressedGradient.clear();

	if(memoryNormalBitmap)
	{
		memoryNormalBitmap = (HBITMAP) SelectObject(memoryNormalDC, memoryNormalBitmap);
		DeleteObject(memoryNormalBitmap);
	}
	if(memoryNormalDC)
		DeleteDC(memoryNormalDC);

	if(memoryActiveBitmap)
	{
		memoryActiveBitmap = (HBITMAP) SelectObject(memoryActiveDC, memoryActiveBitmap);
		DeleteObject(memoryActiveBitmap);
	}
	if(memoryActiveDC)
		DeleteDC(memoryActiveDC);

	if(memoryMinimizedBitmap)
	{
		memoryMinimizedBitmap = (HBITMAP) SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
		DeleteObject(memoryMinimizedBitmap);
	}
	if(memoryMinimizedDC)
		DeleteDC(memoryMinimizedDC);

	if(memoryFlashingBitmap)
	{
		memoryFlashingBitmap = (HBITMAP) SelectObject(memoryFlashingDC, memoryFlashingBitmap);
		DeleteObject(memoryFlashingBitmap);
	}
	if(memoryFlashingDC)
		DeleteDC(memoryFlashingDC);

	if(memoryHoverBitmap)
	{
		memoryHoverBitmap = (HBITMAP) SelectObject(memoryHoverDC, memoryHoverBitmap);
		DeleteObject(memoryHoverBitmap);
	}
	if(memoryHoverDC)
		DeleteDC(memoryHoverDC);

	if(memoryGroupBitmap)
	{
		memoryGroupBitmap = (HBITMAP) SelectObject(memoryGroupDC, memoryGroupBitmap);
		DeleteObject(memoryGroupBitmap);
	}
	if(memoryGroupDC)
		DeleteDC(memoryGroupDC);

	if(memoryPressedBitmap)
	{
		memoryPressedBitmap = (HBITMAP) SelectObject(memoryPressedDC, memoryPressedBitmap);
		DeleteObject(memoryPressedBitmap);
	}
	if(memoryPressedDC)
		DeleteDC(memoryPressedDC);
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor, int imageType)
{
	bgcolor = RGB(255,255,255);
	lightcolor = GetSysColor(COLOR_3DLIGHT);
	darkcolor = GetSysColor(COLOR_3DSHADOW);
		
	if ( cfg.empty() )
		return;

	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);

	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		if (imageType == 0)
		{
			lightcolor = GetSysColor(COLOR_3DLIGHT);
			darkcolor = GetSysColor(COLOR_3DSHADOW);
		}
		else
		{
			lightcolor = normal.lightBevelColor;
			darkcolor = normal.darkBevelColor;
		}
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color;
	COLORREF insertColor;
	char token[4096] = {0};
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 0)
		{
			NormalGradient.insert(NormalGradient.end(), insertColor );
		}
		else if (imageType == 1)
		{
			ActiveGradient.insert(ActiveGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			MinimizedGradient.insert(MinimizedGradient.end(), insertColor );
		}
		else if (imageType == 3)
		{
			FlashingGradient.insert(FlashingGradient.end(), insertColor );
		}
		else if (imageType == 4)
		{
			HoverGradient.insert(HoverGradient.end(), insertColor );
		}
		else if (imageType == 5)
		{
			GroupGradient.insert(GroupGradient.end(), insertColor );
		}
		else if (imageType == 6)
		{
			PressedGradient.insert(PressedGradient.end(), insertColor );
		}
	}
}

void ButtonTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel)
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = {left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void ButtonTexture::subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings)
{	
	workSettings.image = GetxRCString(type, "ButtonImage", defaultSettings.image);

	workSettings.leftEdge = GetxRCInt(type, "ButtonImageLeftEdge", defaultSettings.leftEdge, 0);
	workSettings.topEdge = GetxRCInt(type, "ButtonImageTopEdge", defaultSettings.topEdge, 0);
	workSettings.rightEdge = GetxRCInt(type, "ButtonImageRightEdge", defaultSettings.rightEdge, 0);
	workSettings.bottomEdge = GetxRCInt(type, "ButtonImageBottomEdge", defaultSettings.bottomEdge, 0);

	workSettings.mode = GetxRCNamedValue(type, "ButtonImageMode", modeConstants, defaultSettings.mode);

	workSettings.button_hueColor = GetxRCColor(type, "ButtonHueColor", defaultSettings.button_hueColor);
	workSettings.button_hueIntensity = GetxRCInt(type , "ButtonHueIntensity", defaultSettings.button_hueIntensity, 0, 100);
	workSettings.button_luminanceIntensity = GetxRCInt(type , "ButtonLuminanceIntensity", defaultSettings.button_luminanceIntensity, -100, 100);
	workSettings.button_saturationIntensity = GetxRCInt(type , "ButtonSaturationIntensity", defaultSettings.button_saturationIntensity, 0, 200);

	workSettings.button_mixColor = GetxRCColor(type, "ButtonMixColor", defaultSettings.button_mixColor);
	workSettings.button_mixIntensity = GetxRCInt(type , "ButtonMixIntensity", defaultSettings.button_mixIntensity, 0, 100);

	workSettings.solidColors = GetxRCLine(type, "ButtonSolidColors", defaultSettings.solidColors);
	workSettings.gradientColors = GetxRCLine(type, "ButtonSolidGradientColors", defaultSettings.gradientColors);

	workSettings.bevelsize = GetxRCLine(type, "ButtonSolidBevelSize", defaultSettings.bevelsize);

	workSettings.verticalGradient = GetxRCBool(type, "ButtonSolidGradientVertical", defaultSettings.verticalGradient);
	workSettings.repeatedGradient = GetxRCInt(type, "ButtonSolidGradientRepeated", defaultSettings.repeatedGradient, 0);

	workSettings.gradientType = GetxRCNamedValue(type, "ButtonSolidGradientType", gradientmodeConstants, defaultSettings.gradientType);
	workSettings.transformationType = GetxRCNamedValue(type, "ButtonSolidTransformationType", gradienttransformationConstants, defaultSettings.transformationType);

	workSettings.alphaTransparency = GetxRCInt(type, "ButtonAlphaTransparency", defaultSettings.alphaTransparency, 0, 255);
	workSettings.borderMethod = GetxRCNamedValue(type, "ButtonBorderMethod", borderValues, defaultSettings.borderMethod);
	workSettings.trueTransparent = GetxRCBool(type, "ButtonTrueTransparent", defaultSettings.trueTransparent);
}

void ButtonTexture::configure(const string &name, bool alphaMap)
{
	defaultbutton.image = GetxRCString(name, "ButtonImage", "");

	defaultbutton.leftEdge = GetxRCInt(name, "ButtonImageLeftEdge", 0, 0);
	defaultbutton.topEdge = GetxRCInt(name, "ButtonImageTopEdge", 0, 0);
	defaultbutton.rightEdge = GetxRCInt(name, "ButtonImageRightEdge", 0, 0);
	defaultbutton.bottomEdge = GetxRCInt(name, "ButtonImageBottomEdge", 0, 0);

	defaultbutton.mode = GetxRCNamedValue(name, "ButtonImageMode", modeConstants, MULTIBLT_STRETCH);

	defaultbutton.button_hueColor = GetxRCColor(name, "ButtonHueColor", RGB(255, 255, 255));
	defaultbutton.button_hueIntensity = GetxRCInt(name, "ButtonHueIntensity", 0, 0, 100);
	defaultbutton.button_luminanceIntensity = GetxRCInt(name, "ButtonLuminanceIntensity", 0, -100, 100);
	defaultbutton.button_saturationIntensity = GetxRCInt(name, "ButtonSaturationIntensity", 100, 0, 200);

	defaultbutton.button_mixColor = GetxRCColor(name, "ButtonMixColor", RGB(255, 255, 255));
	defaultbutton.button_mixIntensity = GetxRCInt(name, "ButtonMixIntensity", 0, 0, 100);

	_defaultusesolidcolors = GetxRCBool(name, "UseButtonSolidColors");

	defaultbutton.solidColors = GetxRCLine(name, "ButtonSolidColors", "");
	defaultbutton.gradientColors = GetxRCLine(name, "ButtonSolidGradientColors", "");

	defaultbutton.bevelsize = GetxRCLine(name, "ButtonSolidBevelSize", "0");

	defaultbutton.verticalGradient = GetxRCBool(name, "ButtonSolidGradientVertical");
	defaultbutton.repeatedGradient = GetxRCInt(name, "ButtonSolidGradientRepeated", 0);

	defaultbutton.gradientType = GetxRCNamedValue(name, "ButtonSolidGradientType", gradientmodeConstants, -1);
	defaultbutton.transformationType = GetxRCNamedValue(name, "ButtonSolidTransformationType", gradienttransformationConstants, -1);

	defaultbutton.alphaTransparency = GetxRCInt(name, "ButtonAlphaTransparency", 255, 0, 255);
	defaultbutton.borderMethod = GetxRCNamedValue(name, "ButtonBorderMethod", borderValues, 0);
	defaultbutton.trueTransparent = GetxRCBool(name, "ButtonTrueTransparent");

	useAlphaMap = alphaMap;

	subconfigure((name + "Normal"), normal, defaultbutton);

	subconfigure((name + "Active"), active, defaultbutton);

	subconfigure((name + "Minimized"), minimized, defaultbutton);

	subconfigure((name + "Flashing"), flashing, defaultbutton);

	subconfigure((name + "Hover"), hover, defaultbutton);

	subconfigure((name + "Grouped"), group, defaultbutton);

	subconfigure((name + "Pressed"), pressed, defaultbutton);

	usenormalSolidColors = GetxRCBool(name, "UseNormalButtonSolidColors", _defaultusesolidcolors);

	if (usenormalSolidColors)
	{
		if ( !normal.solidColors.empty() )
		{
			parseCFG(normal.solidColors, normal.backgroundColor, normal.lightBevelColor, normal.darkBevelColor, 0);
		}
		else
		{
			normal.backgroundColor = GetSysColor(COLOR_INACTIVECAPTION);
			normal.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			normal.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		NormalGradient.insert(NormalGradient.end(), normal.backgroundColor );

		if (normal.backgroundColor == RGB(255,0,255) || normal.lightBevelColor == RGB(255,0,255) || normal.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(normal.bevelsize, normal.bevelSizeLeft, normal.bevelSizeRight, normal.bevelSizeTop, normal.bevelSizeBottom);

		parseGradientCFG(normal.gradientColors, 0);
	}
	else
	{
		memoryNormalBitmap = LoadLSImage(normal.image.c_str(), 0);

		if (memoryNormalBitmap)
		{
			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryNormalBitmap, sizeof(BITMAP), &bm);
				
				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				{
					PrepareAlphaChannel(memoryNormalBitmap, normal.button_hueColor, normal.button_hueIntensity, normal.button_luminanceIntensity, normal.button_saturationIntensity, normal.button_mixColor, normal.button_mixIntensity);
				}		
			}

			GetLSBitmapSize(memoryNormalBitmap, &imageNormalWidth, &imageNormalHeight);

			memoryNormalDC = CreateCompatibleDC(NULL);
			SelectObject(memoryNormalDC, memoryNormalBitmap);
		}
		else
		{
			transparencyNeeded = true;
		}
	}

	useactiveSolidColors = GetxRCBool(name, "UseActiveButtonSolidColors", _defaultusesolidcolors);

	if (useactiveSolidColors)
	{
		if ( !active.solidColors.empty() )
		{
			parseCFG(active.solidColors, active.backgroundColor, active.lightBevelColor, active.darkBevelColor, 1);
		}
		else
		{
			active.backgroundColor = GetSysColor(COLOR_ACTIVECAPTION);
			active.lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			active.darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		ActiveGradient.insert(ActiveGradient.end(), active.backgroundColor );

		if (active.backgroundColor == RGB(255,0,255) || active.lightBevelColor == RGB(255,0,255) || active.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(active.bevelsize, active.bevelSizeLeft, active.bevelSizeRight, active.bevelSizeTop, active.bevelSizeBottom);

		if ( active.solidColors.empty() && active.gradientColors.empty() )
			ActiveGradient.insert(ActiveGradient.end(), GetSysColor(COLOR_GRADIENTACTIVECAPTION) );
		else
			parseGradientCFG(active.gradientColors, 1);
	}
	else
	{
		if (!active.image.empty())
		{
			memoryActiveBitmap = LoadLSImage(active.image.c_str(), 0);

			if (memoryActiveBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryActiveBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryActiveBitmap, active.button_hueColor, active.button_hueIntensity, active.button_luminanceIntensity, active.button_saturationIntensity, active.button_mixColor, active.button_mixIntensity);
					}	
				}

				GetLSBitmapSize(memoryActiveBitmap, &imageActiveWidth, &imageActiveHeight);

				memoryActiveDC = CreateCompatibleDC(NULL);
				SelectObject(memoryActiveDC, memoryActiveBitmap);
			}
		}
	}

	useminimizedSolidColors = GetxRCBool(name, "UseMinimizedButtonSolidColors", _defaultusesolidcolors);

	if (useminimizedSolidColors)
	{
		if ( !minimized.solidColors.empty() )
		{
			parseCFG(minimized.solidColors, minimized.backgroundColor, minimized.lightBevelColor, minimized.darkBevelColor, 2);
		}
		else
		{
			minimized.backgroundColor = normal.backgroundColor;
			minimized.lightBevelColor = normal.lightBevelColor;
			minimized.darkBevelColor = normal.darkBevelColor;
		}
		//BG Insert in Gradient List
		MinimizedGradient.insert(MinimizedGradient.end(), minimized.backgroundColor );

		if (minimized.backgroundColor == RGB(255,0,255) || minimized.lightBevelColor == RGB(255,0,255) || minimized.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(minimized.bevelsize, minimized.bevelSizeLeft, minimized.bevelSizeRight, minimized.bevelSizeTop, minimized.bevelSizeBottom);

		parseGradientCFG(minimized.gradientColors, 2);
	}
	else
	{
		if (!minimized.image.empty())
		{
			memoryMinimizedBitmap = LoadLSImage(minimized.image.c_str(), 0);

			if (memoryMinimizedBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryMinimizedBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryMinimizedBitmap, minimized.button_hueColor, minimized.button_hueIntensity, minimized.button_luminanceIntensity, minimized.button_saturationIntensity, minimized.button_mixColor, minimized.button_mixIntensity);
					}		
				}

				GetLSBitmapSize(memoryMinimizedBitmap, &imageMinimizedWidth, &imageMinimizedHeight);

				memoryMinimizedDC = CreateCompatibleDC(NULL);
				SelectObject(memoryMinimizedDC, memoryMinimizedBitmap);
			}
		}
	}

	useflashingSolidColors = GetxRCBool(name, "UseFlashingButtonSolidColors", _defaultusesolidcolors);

	if (useflashingSolidColors)
	{
		if ( !flashing.solidColors.empty() )
		{
			parseCFG(flashing.solidColors, flashing.backgroundColor, flashing.lightBevelColor, flashing.darkBevelColor, 3);
		}
		else
		{
			flashing.backgroundColor = active.backgroundColor;
			flashing.lightBevelColor = active.lightBevelColor;
			flashing.darkBevelColor = active.darkBevelColor;
		}
		//BG Insert in Gradient List
		FlashingGradient.insert(FlashingGradient.end(), flashing.backgroundColor );

		if (flashing.backgroundColor == RGB(255,0,255) || flashing.lightBevelColor == RGB(255,0,255) || flashing.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(flashing.bevelsize, flashing.bevelSizeLeft, flashing.bevelSizeRight, flashing.bevelSizeTop, flashing.bevelSizeBottom);

		parseGradientCFG(flashing.gradientColors, 3);
	}
	else
	{
		if (!flashing.image.empty())
		{
			memoryFlashingBitmap = LoadLSImage(flashing.image.c_str(), 0);

			if (memoryFlashingBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryFlashingBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryFlashingBitmap, flashing.button_hueColor, flashing.button_hueIntensity, flashing.button_luminanceIntensity, flashing.button_saturationIntensity, flashing.button_mixColor, flashing.button_mixIntensity);
					}		
				}

				GetLSBitmapSize(memoryFlashingBitmap, &imageFlashingWidth, &imageFlashingHeight);

				memoryFlashingDC = CreateCompatibleDC(NULL);
				SelectObject(memoryFlashingDC, memoryFlashingBitmap);
			}
		}
	}

	usehoverSolidColors = GetxRCBool(name, "UseHoverButtonSolidColors");

	if (usehoverSolidColors)
	{
		parseCFG(hover.solidColors, hover.backgroundColor, hover.lightBevelColor, hover.darkBevelColor, 4);
	
		//BG Insert in Gradient List
		HoverGradient.insert(HoverGradient.end(), hover.backgroundColor );

		if (hover.backgroundColor == RGB(255,0,255) || hover.lightBevelColor == RGB(255,0,255) || hover.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(hover.bevelsize, hover.bevelSizeLeft, hover.bevelSizeRight, hover.bevelSizeTop, hover.bevelSizeBottom);

		parseGradientCFG(hover.gradientColors, 4);
	}
	else
	{
		if (!hover.image.empty())
		{
			memoryHoverBitmap = LoadLSImage(hover.image.c_str(), 0);

			if (memoryHoverBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryHoverBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryHoverBitmap, hover.button_hueColor, hover.button_hueIntensity, hover.button_luminanceIntensity, hover.button_saturationIntensity, hover.button_mixColor, hover.button_mixIntensity);
					}	
				}

				GetLSBitmapSize(memoryHoverBitmap, &imageHoverWidth, &imageHoverHeight);

				memoryHoverDC = CreateCompatibleDC(NULL);
				SelectObject(memoryHoverDC, memoryHoverBitmap);
			}
		}
	}

	usegroupSolidColors = GetxRCBool(name, "UseGroupedButtonSolidColors");

	if (usegroupSolidColors)
	{
		parseCFG(group.solidColors, group.backgroundColor, group.lightBevelColor, group.darkBevelColor, 5);
	
		//BG Insert in Gradient List
		GroupGradient.insert(GroupGradient.end(), group.backgroundColor );

		if (group.backgroundColor == RGB(255,0,255) || group.lightBevelColor == RGB(255,0,255) || group.darkBevelColor == RGB(255,0,255))
			transparencyNeeded = true; 

		parseBevelCFG(group.bevelsize, group.bevelSizeLeft, group.bevelSizeRight, group.bevelSizeTop, group.bevelSizeBottom);

		parseGradientCFG(group.gradientColors, 5);
	}
	else
	{
		if (!group.image.empty())
		{
			memoryGroupBitmap = LoadLSImage(group.image.c_str(), 0);

			if (memoryGroupBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryGroupBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryGroupBitmap, group.button_hueColor, group.button_hueIntensity, group.button_luminanceIntensity, group.button_saturationIntensity, group.button_mixColor, group.button_mixIntensity);
					}	
				}

				GetLSBitmapSize(memoryGroupBitmap, &imageGroupWidth, &imageGroupHeight);

				memoryGroupDC = CreateCompatibleDC(NULL);
				SelectObject(memoryGroupDC, memoryGroupBitmap);
			}
		}
	}

	usepressedSolidColors = GetxRCBool(name, "UsePressedButtonSolidColors");

	if (usepressedSolidColors)
	{
		parseCFG(pressed.solidColors, pressed.backgroundColor, pressed.lightBevelColor, pressed.darkBevelColor, 6);
	
		//BG Insert in Gradient List
		PressedGradient.insert(PressedGradient.end(), pressed.backgroundColor );

		if (pressed.backgroundColor == RGB(255,0,255) || pressed.lightBevelColor == RGB(255,0,255) || pressed.darkBevelColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		parseBevelCFG(pressed.bevelsize, pressed.bevelSizeLeft, pressed.bevelSizeRight, pressed.bevelSizeTop, pressed.bevelSizeBottom);

		parseGradientCFG(pressed.gradientColors, 6);
	}
	else
	{
		if (!pressed.image.empty())
		{
			memoryPressedBitmap = LoadLSImage(pressed.image.c_str(), 0);

			if (memoryPressedBitmap)
			{
				if (useAlphaMap)
				{
					BITMAP bm;
					GetObject(memoryPressedBitmap, sizeof(BITMAP), &bm);
					
					if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					{
						PrepareAlphaChannel(memoryPressedBitmap, pressed.button_hueColor, pressed.button_hueIntensity, pressed.button_luminanceIntensity, pressed.button_saturationIntensity, pressed.button_mixColor, pressed.button_mixIntensity);
					}	
				}

				GetLSBitmapSize(memoryPressedBitmap, &imagePressedWidth, &imagePressedHeight);

				memoryPressedDC = CreateCompatibleDC(NULL);
				SelectObject(memoryPressedDC, memoryPressedBitmap);
			}
		}
	}

	if ( !useAlphaMap )
	{
		if (memoryNormalDC && !usenormalSolidColors)
			paintEffects(memoryNormalDC, imageNormalWidth, imageNormalHeight, normal.button_hueColor, normal.button_hueIntensity, normal.button_luminanceIntensity, normal.button_saturationIntensity, normal.button_mixColor, normal.button_mixIntensity);
		if (memoryActiveDC && !useactiveSolidColors)
			paintEffects(memoryActiveDC, imageActiveWidth, imageActiveHeight, active.button_hueColor, active.button_hueIntensity, active.button_luminanceIntensity, active.button_saturationIntensity, active.button_mixColor, active.button_mixIntensity);
		if (memoryMinimizedDC && !useminimizedSolidColors)
			paintEffects(memoryMinimizedDC, imageMinimizedWidth, imageMinimizedHeight, minimized.button_hueColor, minimized.button_hueIntensity, minimized.button_luminanceIntensity, minimized.button_saturationIntensity, minimized.button_mixColor, minimized.button_mixIntensity);
		if (memoryFlashingDC && !useflashingSolidColors)
			paintEffects(memoryFlashingDC, imageFlashingWidth, imageFlashingHeight, flashing.button_hueColor, flashing.button_hueIntensity, flashing.button_luminanceIntensity, flashing.button_saturationIntensity, flashing.button_mixColor, flashing.button_mixIntensity);
		if (memoryHoverDC && !usehoverSolidColors)
			paintEffects(memoryHoverDC, imageHoverWidth, imageHoverHeight, hover.button_hueColor, hover.button_hueIntensity, hover.button_luminanceIntensity, hover.button_saturationIntensity, hover.button_mixColor, hover.button_mixIntensity);
		if (memoryGroupDC && !usegroupSolidColors)
			paintEffects(memoryGroupDC, imageGroupWidth, imageGroupHeight, group.button_hueColor, group.button_hueIntensity, group.button_luminanceIntensity, group.button_saturationIntensity, group.button_mixColor, group.button_mixIntensity);
		if (memoryPressedDC && !usepressedSolidColors)
			paintEffects(memoryPressedDC, imagePressedWidth, imagePressedHeight, pressed.button_hueColor, pressed.button_hueIntensity, pressed.button_luminanceIntensity, pressed.button_saturationIntensity, pressed.button_mixColor, pressed.button_mixIntensity);
	}
}

bool ButtonTexture::useAlphaFeatures(int buttonType)
{
	if ( buttonType == 0 && normal.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 1 && active.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 2 && minimized.alphaTransparency != 255 )
		return true;
	else if ( buttonType == 3 && flashing.alphaTransparency != 255 )
		return true;
	else if (buttonType == 4 && hover.alphaTransparency != 255 )
		return true;
	else if (buttonType == 5 && group.alphaTransparency != 255 )
		return true;
	else if (buttonType == 6 && pressed.alphaTransparency != 255 )
		return true;

	return false;
}

void ButtonTexture::apply(HDC hDC, int x, int y, int width, int height, int buttonType, int buttonType2)
{
	if ( width <= 0 || height <= 0 )
		return;
	
	if ( useAlphaFeatures(buttonType) || useAlphaMap )
	{
		//Paint AlphaTransparent Buttons
		applyOverlay(hDC, x, y, width, height, buttonType, buttonType2);
		return;
	}

	if ( buttonType2 != -1 && buttonType != buttonType2 )
	{
		//Paint Normal (buttonType2) State below Hover, Pressed or Grouped State
		applyOverlay(hDC, x, y, width, height, buttonType2);
	}

	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (buttonType == 0 && usenormalSolidColors)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = normal.borderMethod;
		useTrueTransparent = normal.trueTransparent;

		bevelWidth = normal.bevelSizeLeft+normal.bevelSizeRight;
		bevelHeight = normal.bevelSizeTop+normal.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, normal.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normal.bevelSizeRight;
			r.bottom = r.bottom-normal.bevelSizeBottom;
			::SetBkColor(bufferDC, normal.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normal.bevelSizeLeft;
			r.top = r.top+normal.bevelSizeTop;

			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 1 && useactiveSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = active.borderMethod;
		useTrueTransparent = active.trueTransparent;

		bevelWidth = active.bevelSizeLeft+active.bevelSizeRight;
		bevelHeight = active.bevelSizeTop+active.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, active.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-active.bevelSizeRight;
			r.bottom = r.bottom-active.bevelSizeBottom;
			::SetBkColor(bufferDC, active.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+active.bevelSizeLeft;
			r.top = r.top+active.bevelSizeTop;

			if ( ActiveGradient.size() > 1 )
			{
				if ( active.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, ActiveGradient, r, 1);
				else
					DrawGradient(bufferDC, r, ActiveGradient[0], ActiveGradient[1], active.gradientType, active.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, active.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( ActiveGradient.size() > 1 )
			{
				if ( active.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, ActiveGradient, r, 1);
				else
					DrawGradient(bufferDC, r, ActiveGradient[0], ActiveGradient[1], active.gradientType, active.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, active.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 2 && useminimizedSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = minimized.borderMethod;
		useTrueTransparent = minimized.trueTransparent;

		bevelWidth = minimized.bevelSizeLeft+minimized.bevelSizeRight;
		bevelHeight = minimized.bevelSizeTop+minimized.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, minimized.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-minimized.bevelSizeRight;
			r.bottom = r.bottom-minimized.bevelSizeBottom;
			::SetBkColor(bufferDC, minimized.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+minimized.bevelSizeLeft;
			r.top = r.top+minimized.bevelSizeTop;

			if ( MinimizedGradient.size() > 1 )
			{
				if ( minimized.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
				else
					DrawGradient(bufferDC, r, MinimizedGradient[0], MinimizedGradient[1], minimized.gradientType, minimized.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, minimized.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( MinimizedGradient.size() > 1 )
			{
				if ( minimized.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
				else
					DrawGradient(bufferDC, r, MinimizedGradient[0], MinimizedGradient[1], minimized.gradientType, minimized.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, minimized.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 3 && useflashingSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = flashing.borderMethod;
		useTrueTransparent = flashing.trueTransparent;

		bevelWidth = flashing.bevelSizeLeft+flashing.bevelSizeRight;
		bevelHeight = flashing.bevelSizeTop+flashing.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, flashing.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-flashing.bevelSizeRight;
			r.bottom = r.bottom-flashing.bevelSizeBottom;
			::SetBkColor(bufferDC, flashing.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+flashing.bevelSizeLeft;
			r.top = r.top+flashing.bevelSizeTop;

			if ( FlashingGradient.size() > 1 )
			{
				if ( flashing.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FlashingGradient, r, 3);
				else
					DrawGradient(bufferDC, r, FlashingGradient[0], FlashingGradient[1], flashing.gradientType, flashing.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, flashing.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FlashingGradient.size() > 1 )
			{
				if ( flashing.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FlashingGradient, r, 3);
				else
					DrawGradient(bufferDC, r, FlashingGradient[0], FlashingGradient[1], flashing.gradientType, flashing.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, flashing.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 4 && usehoverSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = hover.borderMethod;
		useTrueTransparent = hover.trueTransparent;

		bevelWidth = hover.bevelSizeLeft+hover.bevelSizeRight;
		bevelHeight = hover.bevelSizeTop+hover.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hover.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hover.bevelSizeRight;
			r.bottom = r.bottom-hover.bevelSizeBottom;
			::SetBkColor(bufferDC, hover.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hover.bevelSizeLeft;
			r.top = r.top+hover.bevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 4);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 4);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 5 && usegroupSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = group.borderMethod;
		useTrueTransparent = group.trueTransparent;

		bevelWidth = group.bevelSizeLeft+group.bevelSizeRight;
		bevelHeight = group.bevelSizeTop+group.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, group.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-group.bevelSizeRight;
			r.bottom = r.bottom-group.bevelSizeBottom;
			::SetBkColor(bufferDC, group.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+group.bevelSizeLeft;
			r.top = r.top+group.bevelSizeTop;

			if ( GroupGradient.size() > 1 )
			{
				if ( group.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, GroupGradient, r, 5);
				else
					DrawGradient(bufferDC, r, GroupGradient[0], GroupGradient[1], group.gradientType, group.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, group.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( GroupGradient.size() > 1 )
			{
				if ( group.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, GroupGradient, r, 5);
				else
					DrawGradient(bufferDC, r, GroupGradient[0], GroupGradient[1], group.gradientType, group.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, group.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else if ( buttonType == 6 && usepressedSolidColors )
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		usedBorderMethod = pressed.borderMethod;
		useTrueTransparent = pressed.trueTransparent;

		bevelWidth = pressed.bevelSizeLeft+pressed.bevelSizeRight;
		bevelHeight = pressed.bevelSizeTop+pressed.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, pressed.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-pressed.bevelSizeRight;
			r.bottom = r.bottom-pressed.bevelSizeBottom;
			::SetBkColor(bufferDC, pressed.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+pressed.bevelSizeLeft;
			r.top = r.top+pressed.bevelSizeTop;

			if ( PressedGradient.size() > 1 )
			{
				if ( pressed.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, PressedGradient, r, 6);
				else
					DrawGradient(bufferDC, r, PressedGradient[0], PressedGradient[1], pressed.gradientType, pressed.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, pressed.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( PressedGradient.size() > 1 )
			{
				if ( pressed.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, PressedGradient, r, 6);
				else
					DrawGradient(bufferDC, r, PressedGradient[0], PressedGradient[1], pressed.gradientType, pressed.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, group.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		if (buttonType == 4 && memoryHoverBitmap == 0)
			return;
		else if (buttonType == 5 && memoryGroupBitmap == 0)
			return;
		else if (buttonType == 6 && memoryPressedBitmap == 0)
			return;
	
		if ( buttonType == 1 && memoryActiveBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 2 && memoryMinimizedBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
			buttonType = 0;

		if (buttonType == 0 && memoryNormalBitmap == 0)
			return;

		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		if (buttonType == 0)
		{
			usedBorderMethod = normal.borderMethod;
			useTrueTransparent = normal.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normal.leftEdge, normal.topEdge, normal.rightEdge, normal.bottomEdge,
					normal.mode, SRCCOPY);
		}
		else if (buttonType == 1)
		{
			usedBorderMethod = active.borderMethod;
			useTrueTransparent = active.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryActiveDC, 0, 0, imageActiveWidth, imageActiveHeight,
					active.leftEdge, active.topEdge, active.rightEdge, active.bottomEdge,
					active.mode, SRCCOPY);
		}
		else if (buttonType == 2)
		{
			usedBorderMethod = minimized.borderMethod;
			useTrueTransparent = minimized.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryMinimizedDC, 0, 0, imageMinimizedWidth, imageMinimizedHeight,
					minimized.leftEdge, minimized.topEdge, minimized.rightEdge, minimized.bottomEdge,
					minimized.mode, SRCCOPY);
		}
		else if (buttonType == 3)
		{
			usedBorderMethod = flashing.borderMethod;
			useTrueTransparent = flashing.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFlashingDC, 0, 0, imageFlashingWidth, imageFlashingHeight,
					flashing.leftEdge, flashing.topEdge, flashing.rightEdge, flashing.bottomEdge,
					flashing.mode, SRCCOPY);
		}
		else if (buttonType == 4)
		{
			usedBorderMethod = hover.borderMethod;
			useTrueTransparent = hover.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hover.leftEdge, hover.topEdge, hover.rightEdge, hover.bottomEdge,
					hover.mode, SRCCOPY);
		}
		else if (buttonType == 5)
		{
			usedBorderMethod = group.borderMethod;
			useTrueTransparent = group.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryGroupDC, 0, 0, imageGroupWidth, imageGroupHeight,
					group.leftEdge, group.topEdge, group.rightEdge, group.bottomEdge,
					group.mode, SRCCOPY);
		}
		else if (buttonType == 6)
		{
			usedBorderMethod = pressed.borderMethod;
			useTrueTransparent = pressed.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryPressedDC, 0, 0, imagePressedWidth, imagePressedHeight,
					pressed.leftEdge, pressed.topEdge, pressed.rightEdge, pressed.bottomEdge,
					pressed.mode, SRCCOPY);
		}

		if ( useTrueTransparent )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
								bufferDC, 0, 0,
								RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}

	RECT completeFillRect = {x, y, x+width, y+height};

	if (usedBorderMethod == 1)
		DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
}

void ButtonTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int buttonType, int buttonType2)
{
	if ( width <= 0 || height <= 0 )
		return;

	if ( buttonType2 != -1 && buttonType != buttonType2 )
	{
		//Paint Normal (buttonType2) State below Hover, Pressed or Grouped State
		applyOverlay(hDC, x, y, width, height, buttonType2);
	}

	RECT r;
	r.left = 0;
	r.top = 0;
	r.right = width;
	r.bottom = height;

	int usedAlphaTransparency = 255;
	int usedBorderMethod = 0;
	bool useTrueTransparent = false;

	bool solidColorsApplied = false;

	int bevelWidth = 0;
	int bevelHeight = 0;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (buttonType == 0 && usenormalSolidColors)
	{
		solidColorsApplied = true;

		usedAlphaTransparency = normal.alphaTransparency;
		usedBorderMethod = normal.borderMethod;
		useTrueTransparent = normal.trueTransparent;

		bevelWidth = normal.bevelSizeLeft+normal.bevelSizeRight;
		bevelHeight = normal.bevelSizeTop+normal.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, normal.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-normal.bevelSizeRight;
			r.bottom = r.bottom-normal.bevelSizeBottom;
			::SetBkColor(bufferDC, normal.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+normal.bevelSizeLeft;
			r.top = r.top+normal.bevelSizeTop;

			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( NormalGradient.size() > 1 )
			{
				if ( normal.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, NormalGradient, r, 0);
				else
					DrawGradient(bufferDC, r, NormalGradient[0], NormalGradient[1], normal.gradientType, normal.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, normal.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 1 && useactiveSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = active.alphaTransparency;
		usedBorderMethod = active.borderMethod;
		useTrueTransparent = active.trueTransparent;

		bevelWidth = active.bevelSizeLeft+active.bevelSizeRight;
		bevelHeight = active.bevelSizeTop+active.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, active.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-active.bevelSizeRight;
			r.bottom = r.bottom-active.bevelSizeBottom;
			::SetBkColor(bufferDC, active.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+active.bevelSizeLeft;
			r.top = r.top+active.bevelSizeTop;

			if ( ActiveGradient.size() > 1 )
			{
				if ( active.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, ActiveGradient, r, 1);
				else
					DrawGradient(bufferDC, r, ActiveGradient[0], ActiveGradient[1], active.gradientType, active.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, active.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( ActiveGradient.size() > 1 )
			{
				if ( active.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, ActiveGradient, r, 1);
				else
					DrawGradient(bufferDC, r, ActiveGradient[0], ActiveGradient[1], active.gradientType, active.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, active.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useminimizedSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = minimized.alphaTransparency;
		usedBorderMethod = minimized.borderMethod;
		useTrueTransparent = minimized.trueTransparent;

		bevelWidth = minimized.bevelSizeLeft+minimized.bevelSizeRight;
		bevelHeight = minimized.bevelSizeTop+minimized.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, minimized.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-minimized.bevelSizeRight;
			r.bottom = r.bottom-minimized.bevelSizeBottom;
			::SetBkColor(bufferDC, minimized.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+minimized.bevelSizeLeft;
			r.top = r.top+minimized.bevelSizeTop;

			if ( MinimizedGradient.size() > 1 )
			{
				if ( minimized.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
				else
					DrawGradient(bufferDC, r, MinimizedGradient[0], MinimizedGradient[1], minimized.gradientType, minimized.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, minimized.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( MinimizedGradient.size() > 1 )
			{
				if ( minimized.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, MinimizedGradient, r, 2);
				else
					DrawGradient(bufferDC, r, MinimizedGradient[0], MinimizedGradient[1], minimized.gradientType, minimized.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, minimized.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && useflashingSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = flashing.alphaTransparency;
		usedBorderMethod = flashing.borderMethod;
		useTrueTransparent = flashing.trueTransparent;

		bevelWidth = flashing.bevelSizeLeft+flashing.bevelSizeRight;
		bevelHeight = flashing.bevelSizeTop+flashing.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, flashing.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-flashing.bevelSizeRight;
			r.bottom = r.bottom-flashing.bevelSizeBottom;
			::SetBkColor(bufferDC, flashing.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+flashing.bevelSizeLeft;
			r.top = r.top+flashing.bevelSizeTop;

			if ( FlashingGradient.size() > 1 )
			{
				if ( flashing.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FlashingGradient, r, 3);
				else
					DrawGradient(bufferDC, r, FlashingGradient[0], FlashingGradient[1], flashing.gradientType, flashing.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, flashing.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FlashingGradient.size() > 1 )
			{
				if ( flashing.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, FlashingGradient, r, 3);
				else
					DrawGradient(bufferDC, r, FlashingGradient[0], FlashingGradient[1], flashing.gradientType, flashing.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, flashing.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && usehoverSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = hover.alphaTransparency;
		usedBorderMethod = hover.borderMethod;
		useTrueTransparent = hover.trueTransparent;

		bevelWidth = hover.bevelSizeLeft+hover.bevelSizeRight;
		bevelHeight = hover.bevelSizeTop+hover.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, hover.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-hover.bevelSizeRight;
			r.bottom = r.bottom-hover.bevelSizeBottom;
			::SetBkColor(bufferDC, hover.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+hover.bevelSizeLeft;
			r.top = r.top+hover.bevelSizeTop;

			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 4);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( HoverGradient.size() > 1 )
			{
				if ( hover.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, HoverGradient, r, 4);
				else
					DrawGradient(bufferDC, r, HoverGradient[0], HoverGradient[1], hover.gradientType, hover.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, hover.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 5 && usegroupSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = group.alphaTransparency;
		usedBorderMethod = group.borderMethod;
		useTrueTransparent = group.trueTransparent;

		bevelWidth = group.bevelSizeLeft+group.bevelSizeRight;
		bevelHeight = group.bevelSizeTop+group.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, group.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-group.bevelSizeRight;
			r.bottom = r.bottom-group.bevelSizeBottom;
			::SetBkColor(bufferDC, group.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+group.bevelSizeLeft;
			r.top = r.top+group.bevelSizeTop;

			if ( GroupGradient.size() > 1 )
			{
				if ( group.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, GroupGradient, r, 5);
				else
					DrawGradient(bufferDC, r, GroupGradient[0], GroupGradient[1], group.gradientType, group.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, group.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( GroupGradient.size() > 1 )
			{
				if ( group.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, GroupGradient, r, 5);
				else
					DrawGradient(bufferDC, r, GroupGradient[0], GroupGradient[1], group.gradientType, group.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, group.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 6 && usepressedSolidColors )
	{
		solidColorsApplied = true;

		usedAlphaTransparency = pressed.alphaTransparency;
		usedBorderMethod = pressed.borderMethod;
		useTrueTransparent = pressed.trueTransparent;

		bevelWidth = pressed.bevelSizeLeft+pressed.bevelSizeRight;
		bevelHeight = pressed.bevelSizeTop+pressed.bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, pressed.darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-pressed.bevelSizeRight;
			r.bottom = r.bottom-pressed.bevelSizeBottom;
			::SetBkColor(bufferDC, pressed.lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+pressed.bevelSizeLeft;
			r.top = r.top+pressed.bevelSizeTop;

			if ( PressedGradient.size() > 1 )
			{
				if ( pressed.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, PressedGradient, r, 6);
				else
					DrawGradient(bufferDC, r, PressedGradient[0], PressedGradient[1], pressed.gradientType, pressed.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, pressed.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if (  PressedGradient.size() > 1 )
			{
				if ( pressed.gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC,  PressedGradient, r, 5);
				else
					DrawGradient(bufferDC, r,  PressedGradient[0],  PressedGradient[1], pressed.gradientType, pressed.transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, pressed.backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if (buttonType == 4 && memoryHoverBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if (buttonType == 5 && memoryGroupBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if (buttonType == 6 && memoryPressedBitmap == 0)
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
	
		if ( buttonType == 1 && memoryActiveBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 2 && memoryMinimizedBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}
		else if ( buttonType == 3 && memoryFlashingBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0 && memoryNormalBitmap == 0 )
		{
			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
			return;
		}

		if (buttonType == 0)
		{
			usedAlphaTransparency = normal.alphaTransparency;
			usedBorderMethod = normal.borderMethod;
			useTrueTransparent = normal.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryNormalDC, 0, 0, imageNormalWidth, imageNormalHeight,
					normal.leftEdge, normal.topEdge, normal.rightEdge, normal.bottomEdge,
					normal.mode, SRCCOPY);
		}
		else if (buttonType == 1)
		{
			usedAlphaTransparency = active.alphaTransparency;
			usedBorderMethod = active.borderMethod;
			useTrueTransparent = active.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryActiveDC, 0, 0, imageActiveWidth, imageActiveHeight,
					active.leftEdge, active.topEdge, active.rightEdge, active.bottomEdge,
					active.mode, SRCCOPY);
		}
		else if (buttonType == 2)
		{
			usedAlphaTransparency = minimized.alphaTransparency;
			usedBorderMethod = minimized.borderMethod;
			useTrueTransparent = minimized.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryMinimizedDC, 0, 0, imageMinimizedWidth, imageMinimizedHeight,
					minimized.leftEdge, minimized.topEdge, minimized.rightEdge, minimized.bottomEdge,
					minimized.mode, SRCCOPY);
		}
		else if (buttonType == 3)
		{
			usedAlphaTransparency = flashing.alphaTransparency;
			usedBorderMethod = flashing.borderMethod;
			useTrueTransparent = flashing.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFlashingDC, 0, 0, imageFlashingWidth, imageFlashingHeight,
					flashing.leftEdge, flashing.topEdge, flashing.rightEdge, flashing.bottomEdge,
					flashing.mode, SRCCOPY);
		}
		else if (buttonType == 4)
		{
			usedAlphaTransparency = hover.alphaTransparency;
			usedBorderMethod = hover.borderMethod;
			useTrueTransparent = hover.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryHoverDC, 0, 0, imageHoverWidth, imageHoverHeight,
					hover.leftEdge, hover.topEdge, hover.rightEdge, hover.bottomEdge,
					hover.mode, SRCCOPY);
		}
		else if (buttonType == 5)
		{
			usedAlphaTransparency = group.alphaTransparency;
			usedBorderMethod = group.borderMethod;
			useTrueTransparent = group.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryGroupDC, 0, 0, imageGroupWidth, imageGroupHeight,
					group.leftEdge, group.topEdge, group.rightEdge, group.bottomEdge,
					group.mode, SRCCOPY);
		}
		else if (buttonType == 6)
		{
			usedAlphaTransparency = pressed.alphaTransparency;
			usedBorderMethod = pressed.borderMethod;
			useTrueTransparent = pressed.trueTransparent;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryPressedDC, 0, 0, imagePressedWidth, imagePressedHeight,
					pressed.leftEdge, pressed.topEdge, pressed.rightEdge, pressed.bottomEdge,
					pressed.mode, SRCCOPY);
		}
	}

	RECT completeFillRect = {0, 0, width, height};

	if (usedBorderMethod == 1)
		DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 2)
		DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 3)
		DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (usedBorderMethod == 4)
		DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	
	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap || pPixel[3] == 0 || solidColorsApplied )
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, usedAlphaTransparency, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(r, g, b, hueh, hues, huel);
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				if ( needHSLConversion )
				{
					RGB2HSL(r, g, b, h, s, l);

					if(button_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
					if(button_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

					if(button_saturationIntensity != 100)
					{
						if ( button_saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
					}
				
					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix effect
					r = (BYTE)((r * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
				}
			}
			else
			{
				transparencyNeeded = true;
				if ( !needHSLConversion && !needMixColor )
					return;
			}
	
			if ( needHSLConversion || needMixColor )
				SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, int imageType)
{
	bool vertical = false;
	int repeated = 0;

	if ( imageType == 0 )
	{
		vertical = normal.verticalGradient;
		repeated = normal.repeatedGradient;
	}
	else if( imageType == 1 )
	{
		vertical = active.verticalGradient;
		repeated = active.repeatedGradient;
	}
	else if( imageType == 2 )
	{
		vertical = minimized.verticalGradient;
		repeated = minimized.repeatedGradient;
	}
	else if( imageType == 3 )
	{
		vertical = flashing.verticalGradient;
		repeated = flashing.repeatedGradient;
	}
	else if( imageType == 4 )
	{
		vertical = hover.verticalGradient;
		repeated = hover.repeatedGradient;
	}
	else if( imageType == 5 )
	{
		vertical = group.verticalGradient;
		repeated = group.repeatedGradient;
	}
	else if( imageType == 6 )
	{
		vertical = pressed.verticalGradient;
		repeated = pressed.repeatedGradient;
	}
	
	if ( repeated == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = workGradient.size();

		if (vertical)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
		{
			if (cit == workGradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (vertical)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (vertical)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (vertical)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
			{
				color = *cit;

				if (vertical)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeated, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeated, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (vertical)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

void ButtonTexture::DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType)
{
	int i, j, y, x1, y1;
	double r, d, n1;

	// Calculate image size
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	int size = height*width*4;
	double radius = sqrt( pow(height/2,2) + pow(width/2,2) );
	double k = double(rect.bottom-rect.top)/double(rect.right-rect.left);
	if ( gradientType == GRADIENT_BDIAGONAL )
		k = double(rect.bottom-rect.top)/double(rect.left-rect.right);
	if ( ( gradientType == GRADIENT_FDIAGONAL ) || ( gradientType == GRADIENT_BDIAGONAL ) )
		gradientType = GRADIENT_DIAGONAL;
	double n = 0.0;
	double k1 = -1/k;

	// Create data buffer
	m_Data = new unsigned char[size];
	
	// Create gradient
	for ( i=0; i<height; i++ )
	{
		for ( j=0; j<width; j++ )
		{
			switch (gradientType)
			{
				// Horizontal gradient
				case GRADIENT_HORIZONTAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)j/(double)width)*GetBValue(endColor) + (1-(double)j/(double)width)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)j/(double)width)*GetGValue(endColor) + (1-(double)j/(double)width)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)j/(double)width)*GetRValue(endColor) + (1-(double)j/(double)width)*GetRValue(startColor));
				}
				break;

				// Vertical gradient
				case GRADIENT_VERTICAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)i/(double)height)*GetBValue(endColor) + (1-(double)i/(double)height)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)i/(double)height)*GetGValue(endColor) + (1-(double)i/(double)height)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)i/(double)height)*GetRValue(endColor) + (1-(double)i/(double)height)*GetRValue(startColor));
				}
				break;

				// Radial gradient
				case GRADIENT_RADIAL:
				{
					r = sqrt( pow(double(i-height/2)/radius,2) + pow(double(j-width/2)/radius,2) );
					if ( r <= 1.0 )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)(r*GetBValue(endColor) + (1-r)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)(r*GetGValue(endColor) + (1-r)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)(r*GetRValue(endColor) + (1-r)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = GetBValue(endColor);
						m_Data[i*width*4 + j*4+1] = GetGValue(endColor);
						m_Data[i*width*4 + j*4+2] = GetRValue(endColor);
					}
				}
				break;

				// Diagonal gradient
				case GRADIENT_DIAGONAL:
				{
					y = int(k*(j-width/2) + n);
					n1 = int((i-height/2) - k1*(j-width/2));
					x1 = int((n1-n)/(k-k1));
					y1 = int(k1*x1 + n1);
					d = sqrt( pow((j-width/2)-x1,2) + pow((i-height/2)-y1,2) ) / (2*radius);
					if ( (i-height/2) > y )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(endColor) + (0.5+d)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(endColor) + (0.5+d)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(endColor) + (0.5+d)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(startColor) + (0.5+d)*GetBValue(endColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(startColor) + (0.5+d)*GetGValue(endColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(startColor) + (0.5+d)*GetRValue(endColor));
					}
				}
				break;
			}
		}
	}

	if ( transformationType != TRANSFORMATION_NONE )
		ApplyTransformation( width, height, transformationType );

	// Create temporary bitmap
	HBITMAP hBitmap = (HBITMAP)CreateBitmap( width, height, 1, 32, m_Data );

	// Create temporary device context
	HDC hMemDC = CreateCompatibleDC(hDC);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// Blit gradient bitmap
	BitBlt( hDC, rect.left, rect.top, width, height, hMemDC, 0, 0, SRCCOPY );

	// Free memory
	SelectObject( hMemDC, hOldBitmap );
	DeleteDC(hMemDC);
	DeleteObject(hBitmap);
	delete m_Data;
	m_Data = NULL;
}

void ButtonTexture::ApplyTransformation(int width, int height, int transformationType)
{
	int X, Y;
	double r, fi;
	double R = sqrt( pow(width/4,2) + pow(height/4,2) );
	double FI = atan( double(height)/double(width) );

	int size = height*width*4;
	unsigned char* newBuffer = new unsigned char[size];
	memcpy( newBuffer, m_Data, size );

	for ( int i=0; i<height; i++ )
	{
		for ( int j=0; j<width; j++ )
		{
			switch (transformationType)
			{
				case TRANSFORMATION_CHARICATURE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(sqrt(r*R*2)*cos(fi));
					Y = int(fabs(sqrt(r*R*2)*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_FISHEYE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int((pow(r,2)/R)*cos(fi));
					Y = int(fabs((pow(r,2)/R)*sin(fi))) % height;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SWIRLED:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi+r/8)));
					Y = int(fabs(r*sin(fi+r/8)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_CYLINDER:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = abs((j-width/2) + int(fi+r/10)%32 - 8);
					Y = i;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SHIFT:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi)));
					Y = int(fabs(r*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;
			}
		}
	}

	memcpy( m_Data, newBuffer, size );
	delete newBuffer;
}

void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp, COLORREF button_hueColor, int button_hueIntensity, int button_luminanceIntensity, int button_saturationIntensity, COLORREF button_mixColor, int button_mixIntensity)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( button_luminanceIntensity != 0 || button_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( button_hueIntensity != 0 )
	{
		r = GetRValue(button_hueColor);
		g = GetGValue(button_hueColor);
		b = GetBValue(button_hueColor);

		if ( r == b && b == g )
		{
			button_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( button_mixIntensity != 0 )
	{
		button_mixIntensity = (int) min(255, button_mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int i = 0; i < iSize; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		if ( needHSLConversion )
		{
			//AlphaMap Colorizing
			RGB2HSL(r, g, b, h, s, l);

			if(button_hueIntensity != 0)
				h = (BYTE) max(0, min(240, (h*(100-button_hueIntensity)+hueh*button_hueIntensity)/100));
		
			if(button_luminanceIntensity != 0)
				l = (BYTE) max(0, min(240, ((l*100)+(120*button_luminanceIntensity))/100));

			if(button_saturationIntensity != 100)
			{
				if ( button_saturationIntensity > 100 )
					s = (BYTE) max(0, min(240, (240*(button_saturationIntensity-100))/100));				
				else
					s = (BYTE) max(0, min(240, (s*button_saturationIntensity)/100));
			}

			HSL2RGB(r, g, b, h, s, l);
		}

		if ( needMixColor )
		{
			// HueMix effect
			r = (BYTE)((r * (255 - button_mixIntensity) + GetBValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - button_mixIntensity) + GetGValue(button_mixColor) * button_mixIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - button_mixIntensity) + GetRValue(button_mixColor) * button_mixIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}