/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "TaskbarButton.h"
#include "Taskbar.h"
#include "WindowUtilities.h"
#include "ButtonTexture.h"
#include "ButtonIcon.h"
#include "ButtonFont.h"

#include "GhostButton.h"

//Grouping
#define	X_GROUP_MINIMIZE	10
#define	X_GROUP_RESTORE		11
#define	X_GROUP_MAXIMIZE	12
#define	X_GROUP_CLOSE		13

#define	X_GROUP_SEPARATOR	66

extern int realMonitors;

TaskbarButton::TaskbarButton( Taskbar *actualtaskbar, HWND hWnd ) : 
	taskbar(NULL),
	buttonx(0),
	buttony(0),
	buttonheight(0),
	buttonwidth(0),
	flashState(false),
	isFlashing(false),
	isHover(false),
	isPressed(false),
	isScrolling(false),

	currentState(STATE_NORMAL),
	
	position(-1),

	icon(NULL),

	iconDC(0),
	iconBitmap(0),

	monitorHidden(false),
	currentMonitor(NULL)
{
	taskbar = actualtaskbar;

	hButtonTask = hWnd;

	classname = GetTaskFileName( hWnd );

	hwndgrouplist.insert(hwndgrouplist.end(), hWnd);

	if (taskbar->showText || taskbar->showToolTip)
		caption = GetWindowCaption();
	else
		caption = "";
	
	if (taskbar->showToolTip)
		createTooltip();
	else
		tooltip = "";
}

TaskbarButton::~TaskbarButton()
{
	hwndgrouplist.clear();

	if (icon)
		DestroyIcon(icon);

	if(iconBitmap)
	{
		iconBitmap = (HBITMAP) SelectObject(iconDC, iconBitmap);
		DeleteObject(iconBitmap);
	}
	if(iconDC)
		DeleteDC(iconDC);
}

bool TaskbarButton::HasPoint( int xP, int yP )
{
	if ( xP < (buttonx + buttonwidth) && xP >= buttonx )
	{
		if ( yP < (buttony + buttonheight) && yP >= buttony )
			return TRUE;
	}

	return FALSE;
}


void TaskbarButton::createTooltip(bool grouped)
{
	if ( grouped )
	{
		if ( taskbar->tooltipMode == 0 )
		{
			tooltip = ( "Grouping:\t" + IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( "Active Caption:\t" + caption + "\nExecutable:\t" + classname );
		}
		else if ( taskbar->tooltipMode == 1 )
		{
			tooltip = ( IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( caption + "\n" + classname );
		}
		else if ( taskbar->tooltipMode == 2 )
		{
			tooltip = ( "Grouping:\t" + IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( "Active Caption:\t" + caption );
		}
		else if ( taskbar->tooltipMode == 3 )
		{
			tooltip = ( IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( caption );
		}
		else if ( taskbar->tooltipMode == 4 )
		{
			tooltip = ( "Grouping:\t" + IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( "Executable:\t" + classname );
		}
		else if ( taskbar->tooltipMode == 5 )
		{
			tooltip = ( IntToString(hwndgrouplist.size()) + " Grouped Tasks\n");
			tooltip.append( classname );
		}
	}
	else
	{
		if ( taskbar->tooltipMode == 0 )
		{
			tooltip = ( "Caption:\t" + caption + "\nExecutable:\t" + classname );
		}
		else if ( taskbar->tooltipMode == 1 )
		{
			tooltip = ( caption + "\n" + classname );
		}
		else if ( taskbar->tooltipMode == 2 )
		{
			tooltip = ( "Caption:  " + caption );
		}
		else if ( taskbar->tooltipMode == 3 )
		{
			tooltip = ( caption );
		}
		else if ( taskbar->tooltipMode == 4 )
		{
			tooltip = ( "Executable:  " + classname );
		}
		else if ( taskbar->tooltipMode == 5 )
		{
			tooltip = ( classname );
		}
	}
}

bool TaskbarButton::onUpdate(bool needRedraw)
{
	if ( realMonitors > 1 && taskbar->ThisMultiMonitorOnly != -1 )
	{
		if ( taskbar->ThisMultiMonitorOnly == 0 )
		{
			if ( MonitorFromWindow( taskbar->gethWnd(), MONITOR_DEFAULTTOPRIMARY ) != MonitorFromWindow( hButtonTask, MONITOR_DEFAULTTOPRIMARY ) )
				monitorHidden = true;
			else
				monitorHidden = false;
		}
		else if ( taskbar->ThisMultiMonitorOnly == 1 )
		{
			if ( pMon != MonitorFromWindow( hButtonTask, MONITOR_DEFAULTTOPRIMARY ) )
				monitorHidden = true;
			else
				monitorHidden = false;
		}
		else if ( taskbar->ThisMultiMonitorOnly == 2 )
		{
			if ( sMon != MonitorFromWindow( hButtonTask, MONITOR_DEFAULTTOPRIMARY ) )
				monitorHidden = true;
			else
				monitorHidden = false;
		}
		else if ( taskbar->ThisMultiMonitorOnly == 3 )
		{
			if ( tMon != MonitorFromWindow( hButtonTask, MONITOR_DEFAULTTOPRIMARY ) )
				monitorHidden = true;
			else
				monitorHidden = false;
		}
		else
		{
			monitorHidden = false;
		}

		needRedraw = true;
	}

	if (taskbar->showText || taskbar->showToolTip)
	{
		string temp1 = caption;
		string temp2 = GetWindowCaption();

		if ( _stricmp( temp1.c_str(), temp2.c_str() ) != NULL && !temp2.empty() )
		{
			caption = temp2;
			if (taskbar->showToolTip && hwndgrouplist.size() > 1)
			{
				createTooltip(true);
			}
			else if (taskbar->showToolTip)
			{	
				createTooltip();
			}

			if (taskbar->showText)
				needRedraw = true;
		}
	}

	return needRedraw;	
}

string TaskbarButton::GetWindowCaption( HWND check_hTask )
{
	if (check_hTask == NULL)
		check_hTask = hButtonTask;
	
	return GetTaskWindowText(check_hTask);
}

void TaskbarButton::onButtonClick( int x, int y, int action, bool minimize )
{
	switch (action)
	{
		case 1:
			
			Select(minimize);
		    break;

		case 2:

			Menu();
		    break;

		case 3:

			Close(hButtonTask);
		    break;

		case 4:

			OnTopToggle(hButtonTask);
			break;
	}
}

void TaskbarButton::Select(bool minimize)
{
	if( isFlashing )
	{
		KillTimer(taskbar->gethWnd(), 9/*TIMER_FLASH*/);
		SetFlashState(0);
		currentState = getCurrentState();
	}

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{
		GroupMenu();
		return;
	}

	if ( minimize )
	{
		if (!taskbar->noMinimizeOnClick)
		{
			currentState = STATE_MINIMIZED;
			MinimizeWindow(hButtonTask);
			Sleep(10);
			taskbar->update(true);
		}
		else
		{
			SelectWindow(hButtonTask);
			if ( !taskbar->taskbarSelect.empty() )
				LSExecute(taskbar->gethWnd(), taskbar->taskbarSelect.c_str(), SW_SHOWNORMAL);
		}
	}
	else
	{
		switch(currentState)
		{
			case STATE_SELECTED:

				if (!taskbar->noMinimizeOnClick)
				{
					currentState = STATE_MINIMIZED;
					MinimizeWindow(hButtonTask);
					Sleep(10);
					taskbar->update(true);
				}
				else
				{
					SelectWindow(hButtonTask);
					if ( !taskbar->taskbarSelect.empty() )
						LSExecute(taskbar->gethWnd(), taskbar->taskbarSelect.c_str(), SW_SHOWNORMAL);
				}

			break;

			case STATE_MINIMIZED:

				SelectWindow(hButtonTask, true);

			break;

			default:

				SelectWindow(hButtonTask);
				if ( !taskbar->taskbarSelect.empty() )
					LSExecute(taskbar->gethWnd(), taskbar->taskbarSelect.c_str(), SW_SHOWNORMAL);

			break;
		}
	}
}

void TaskbarButton::Menu()
{
	HMENU systemMenu;

	systemMenu = GetSystemMenu( hButtonTask, FALSE );

	WINDOWPLACEMENT wp;
	wp.length = sizeof(WINDOWPLACEMENT);
	GetWindowPlacement( hButtonTask, &wp );

	// restore is enabled only if minimized or maximized (not normal)
	EnableMenuItem( systemMenu, SC_RESTORE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// move is enabled only if normal
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// size is enabled only if normal
	EnableMenuItem( systemMenu, SC_SIZE, MF_BYCOMMAND | (wp.showCmd == SW_SHOWNORMAL ? MF_ENABLED : MF_GRAYED) );

	// minimize is enabled only if not minimized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMINIMIZED ? MF_ENABLED : MF_GRAYED) );

	// maximize is enabled only if not maximized
	EnableMenuItem( systemMenu, SC_MOVE, MF_BYCOMMAND | (wp.showCmd != SW_SHOWMAXIMIZED ? MF_ENABLED : MF_GRAYED) );

	int counter = 0;

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{	
		AppendMenu( systemMenu, MF_SEPARATOR, X_GROUP_SEPARATOR, "" );

		AppendMenu( systemMenu, MF_STRING, X_GROUP_MINIMIZE, "Minimize All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_RESTORE, "Restore All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_MAXIMIZE, "Maximize All In Group" );
		AppendMenu( systemMenu, MF_STRING, X_GROUP_CLOSE, "Close All In Group" );

		AppendMenu( systemMenu, MF_SEPARATOR, X_GROUP_SEPARATOR, "" );

		counter = 1;
		for(hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit)
		{
			AppendMenu( systemMenu, MF_STRING , counter, ("Close " + GetWindowCaption(*hglit)).c_str() );
			counter++;
		}
		counter--;
		//END Grouping
	}

	// let application modify menu
	PostMessage( hButtonTask, WM_INITMENUPOPUP, (WPARAM) systemMenu, MAKELPARAM( 0, TRUE ) );
	PostMessage( hButtonTask, WM_INITMENU, (WPARAM) systemMenu, 0 );

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( systemMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              taskbar->gethWnd(),
	                              NULL );

	if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
	{
		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_SEPARATOR );

		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_MINIMIZE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_RESTORE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_MAXIMIZE, MF_BYCOMMAND );
		DeleteMenu( systemMenu, X_GROUP_CLOSE, MF_BYCOMMAND );

		DeleteMenu( systemMenu, X_GROUP_SEPARATOR, MF_SEPARATOR );

		while ( counter >= 1 )
		{
			DeleteMenu( systemMenu, counter, MF_BYCOMMAND );
			counter--;
		}
	}

	if ( command )
	{
		if (hwndgrouplist.size() > 1 && taskbar->useGrouping)
		{
			switch( command )
			{
				case X_GROUP_MINIMIZE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						MinimizeWindow( (*hglit) );
					
					return;
				}

				case X_GROUP_RESTORE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						RestoreWindow( (*hglit) );

					return;
				}

				case X_GROUP_MAXIMIZE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						MaximizeWindow( (*hglit) );

					return;
				}

				case X_GROUP_CLOSE:
				{
					for ( hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit )
						Close( (*hglit) );
						
					return;
				}
			}

			hWndGroupListIterator hglit = hwndgrouplist.begin();
			counter = 1; 
			while (counter < command)
			{
				hglit++;
				counter++;
			}

			if ( command <= hwndgrouplist.size() )
			{
				Close( (*hglit) );
				hButtonTask = (*hglit);
				return;
			}
		}
		
		if ( command != SC_MINIMIZE && command != SC_CLOSE )
		{
			SelectWindow(hButtonTask);
			if ( !taskbar->taskbarSelect.empty() )
				LSExecute(taskbar->gethWnd(), taskbar->taskbarSelect.c_str(), SW_SHOWNORMAL);
		}

		PostMessage( hButtonTask, WM_SYSCOMMAND, (WPARAM) command, MAKELPARAM( pt.x, pt.y ) );
	}
}

void TaskbarButton::GroupMenu()
{
	HMENU GroupMenu = CreatePopupMenu();

	int counter = 1;
	for(hWndGroupListIterator hglit = hwndgrouplist.begin(); hglit != hwndgrouplist.end(); ++hglit)
	{
		AppendMenu( GroupMenu, MF_STRING, counter, GetWindowCaption((*hglit)).c_str() );
		counter++;
	}

	// display the menu
	POINT pt;
	GetCursorPos( &pt );

	int command = TrackPopupMenu( GroupMenu,
	                              TPM_RETURNCMD | TPM_RIGHTBUTTON,
	                              pt.x, pt.y, 0,
	                              taskbar->gethWnd(),
	                              NULL );

	if ( command )
	{
		hWndGroupListIterator hglit = hwndgrouplist.begin();
		counter = 1; 
		while (counter < command)
		{
			hglit++;
			counter++;
		}

		if ( counter <= hwndgrouplist.size() )
		{
			if (!taskbar->noMinimizeOnClick && taskbar->lastActiveForGrouping == (*hglit))
			{
				
				MinimizeWindow((*hglit));
			}
			else
			{
				SelectWindow((*hglit), IsIconic((*hglit)));
				if ( !taskbar->taskbarSelect.empty() )
					LSExecute(taskbar->gethWnd(), taskbar->taskbarSelect.c_str(), SW_SHOWNORMAL);

				hButtonTask = (*hglit);
			}
		}
	}
	
	if (GroupMenu)
		DestroyMenu(GroupMenu);
}

int TaskbarButton::getCurrentState()
{
	return GetState(hButtonTask);
}

void TaskbarButton::setButtonLayout( int x, int y, int width, int height )
{
	buttonx = x;
	buttony = y;
	buttonwidth = width;	
	buttonheight = height;
}

void TaskbarButton::onPaint( HDC &destDC )
{
	if ( buttonwidth <= 0 || buttonheight <= 0 )
		return;

	int x = 0;
	int y = 0;
	int width = 0;
	int height = 0;

	//Update Icon - Sure is Sure
	if ( taskbar->showIcon )
	{
		if (icon)
		{
			DestroyIcon(icon);
			icon = NULL;
		}

		if ( GetIconOnCustomList( hButtonTask, icon, taskbar->CustomIconList, taskbar->useBigIcon) )
			icon = (HICON)CopyImage((HANDLE)icon, IMAGE_ICON, 0, 0, 0);
	}

	if (!isFlashing)
	{
		currentState = getCurrentState();

		if( STATE_SELECTED == currentState )
		{
			taskbar->lastActive = hButtonTask;
		}
	}

	if ( isFlashing && !flashState )
		currentState = STATE_NORMAL;
	else if ( isFlashing && flashState )
		currentState = STATE_FLASHING;

	//Button Background Painting with Icon and Text 
	if (currentState == STATE_FLASHING)
	{
		if ( taskbar->isAlphaMap() )
			taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, 3);
		else
			taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 3);
		
		OverlayButtonPainter(destDC, buttonx, buttony, buttonwidth, buttonheight, 3);

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 3, 3, icon, taskbar->isAlphaMap());
		}
		if (taskbar->showText)
		{	
			taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 3, 3);
		}
	}
	else if (taskbar->useGrouping && hwndgrouplist.size() > 1 && !isHover && !isPressed)
	{
		int tmpButtonTextureType = 5;
	
		if (currentState == STATE_SELECTED)
			tmpButtonTextureType = 1;
		else if (currentState == STATE_MINIMIZED)
			tmpButtonTextureType = 2;
		else
			tmpButtonTextureType = 0;
		
		if ( taskbar->isAlphaMap() )
			taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->useGroupedBG() ? 5 : tmpButtonTextureType), tmpButtonTextureType );
		else
			taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->useGroupedBG() ? 5 : tmpButtonTextureType), tmpButtonTextureType );

		OverlayButtonPainter(destDC, buttonx, buttony, buttonwidth, buttonheight, 3);

		if ( !taskbar->ghostbuttonList.empty() )
		{
			for(GhostButtonListIterator glit = taskbar->ghostbuttonList.begin(); glit != taskbar->ghostbuttonList.end(); glit++)
			{
				if ( *glit != NULL )
				{
					x = buttonx + xParseCoord((*glit)->orgx, 0, buttonwidth, taskbar->gethWnd());
					y = buttony + xParseCoord((*glit)->orgy, 0, buttonheight, taskbar->gethWnd());
					width = xParseDimen((*glit)->orgwidth, buttonwidth, buttonwidth, taskbar->gethWnd());
					height = xParseDimen((*glit)->orgheight, buttonheight, buttonheight, taskbar->gethWnd());

					(*glit)->buttontexture->applyOverlay(destDC, x, y, width, height, ((*glit)->buttontexture->useGroupedBG() ? 5 : tmpButtonTextureType), tmpButtonTextureType );
				}
				else
				{
					break;
				}
			}
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 5, tmpButtonTextureType, icon, taskbar->isAlphaMap());
		}
		if (taskbar->showText)
		{
			CString text = caption.c_str();
		
			int found = text.Find(" - ");
			if ( found != -1 && text.Find(" - ", found+3) != -1)
				text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
			else if ( found != -1 )
				text = text.Left(text.Find(" - "));
			text += " ( ";
			text += IntToString( hwndgrouplist.size() ).c_str();
			text += " )";

			taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 5, tmpButtonTextureType);
		}
	}
	else if (isPressed)
	{
		int tmpButtonTextureType = 6;
		
		if (currentState == STATE_SELECTED)
			tmpButtonTextureType = 1;
		else if (currentState == STATE_MINIMIZED)
			tmpButtonTextureType = 2;
		else
			tmpButtonTextureType = 0;
	
		if ( taskbar->isAlphaMap() )
			taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->usePressedBG() ? 6 : tmpButtonTextureType), tmpButtonTextureType );
		else
			taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->usePressedBG() ? 6 : tmpButtonTextureType), tmpButtonTextureType );

		if ( !taskbar->ghostbuttonList.empty() )
		{
			for(GhostButtonListIterator glit = taskbar->ghostbuttonList.begin(); glit != taskbar->ghostbuttonList.end(); glit++)
			{
				if ( *glit != NULL )
				{
					x = buttonx + xParseCoord((*glit)->orgx, 0, buttonwidth, taskbar->gethWnd());
					y = buttony + xParseCoord((*glit)->orgy, 0, buttonheight, taskbar->gethWnd());
					width = xParseDimen((*glit)->orgwidth, buttonwidth, buttonwidth, taskbar->gethWnd());
					height = xParseDimen((*glit)->orgheight, buttonheight, buttonheight, taskbar->gethWnd());

					(*glit)->buttontexture->applyOverlay(destDC, x, y, width, height, ((*glit)->buttontexture->usePressedBG() ? 6 : tmpButtonTextureType), tmpButtonTextureType);
				}
				else
				{
					break;
				}
			}
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 6, tmpButtonTextureType, icon, taskbar->isAlphaMap());
		}
		if (taskbar->showText)
		{
			if (taskbar->useGrouping && hwndgrouplist.size() > 1)
			{
				CString text = caption.c_str();
			
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1)
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";

				taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 6, tmpButtonTextureType);
			}
			else
			{
				taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 6, tmpButtonTextureType);
			}
		}
	}
	else if (isHover || isScrolling)
	{
		int tmpButtonTextureType = 4;
		
		if (currentState == STATE_SELECTED)
			tmpButtonTextureType = 1;
		else if (currentState == STATE_MINIMIZED)
			tmpButtonTextureType = 2;
		else
			tmpButtonTextureType = 0;
	
		if ( taskbar->isAlphaMap() )
			taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->useHoverBG() ? 4 : tmpButtonTextureType), tmpButtonTextureType );
		else
			taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (taskbar->buttontexture->useHoverBG() ? 4 : tmpButtonTextureType), tmpButtonTextureType );

		if ( !taskbar->ghostbuttonList.empty() )
		{
			for(GhostButtonListIterator glit = taskbar->ghostbuttonList.begin(); glit != taskbar->ghostbuttonList.end(); glit++)
			{
				if ( *glit != NULL )
				{
					x = buttonx + xParseCoord((*glit)->orgx, 0, buttonwidth, taskbar->gethWnd());
					y = buttony + xParseCoord((*glit)->orgy, 0, buttonheight, taskbar->gethWnd());
					width = xParseDimen((*glit)->orgwidth, buttonwidth, buttonwidth, taskbar->gethWnd());
					height = xParseDimen((*glit)->orgheight, buttonheight, buttonheight, taskbar->gethWnd());

					(*glit)->buttontexture->applyOverlay(destDC, x, y, width, height, ((*glit)->buttontexture->useHoverBG() ? 4 : tmpButtonTextureType), tmpButtonTextureType );
				}
				else
				{
					break;
				}
			}
		}

		if(taskbar->showIcon)
		{
			taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 4, tmpButtonTextureType, icon, taskbar->isAlphaMap());
		}
		if (taskbar->showText)
		{
			if (taskbar->useGrouping && hwndgrouplist.size() > 1)
			{
				CString text = caption.c_str();
			
				int found = text.Find(" - ");
				if ( found != -1 && text.Find(" - ", found+3) != -1)
					text = text.Mid(found+3, text.Find(" - ", found+3)-found-3);
				else if ( found != -1 )
					text = text.Left(text.Find(" - "));
				text += " ( ";
				text += IntToString( hwndgrouplist.size() ).c_str();
				text += " )";

				if ( isScrolling )
					taskbar->buttonfont->applyScroll(destDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 4, tmpButtonTextureType);
				else
					taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, (string) text, 4, tmpButtonTextureType);
			}
			else
			{
				if ( isScrolling )
					taskbar->buttonfont->applyScroll(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 4, tmpButtonTextureType);
				else
					taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 4, tmpButtonTextureType);
			}
		}
	}
	else
	{
		if (currentState == STATE_SELECTED)
		{
			if ( taskbar->isAlphaMap() )
				taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, 1);
			else
				taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 1);
			
			OverlayButtonPainter(destDC, buttonx, buttony, buttonwidth, buttonheight, 1);

			if(taskbar->showIcon)
			{
				taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 1, 1, icon, taskbar->isAlphaMap());
			}
			if (taskbar->showText)
			{
				taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 1, 1);
			}
		}
		else if (currentState == STATE_MINIMIZED)
		{
			if ( taskbar->isAlphaMap() )
				taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, 2);
			else
				taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 2);
			
			OverlayButtonPainter(destDC, buttonx, buttony, buttonwidth, buttonheight, 2);

			if(taskbar->showIcon)
			{
				taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 2, 2, icon, taskbar->isAlphaMap());
			}
			if (taskbar->showText)
			{
				taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 2, 2);
			}
		}
		else
		{
			if ( taskbar->isAlphaMap() )
				taskbar->buttontexture->applyOverlay(destDC, buttonx, buttony, buttonwidth, buttonheight, 0);
			else
				taskbar->buttontexture->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 0);

			OverlayButtonPainter(destDC, buttonx, buttony, buttonwidth, buttonheight, 0);

			if(taskbar->showIcon)
			{
				taskbar->buttonicon->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, 0, 0, icon, taskbar->isAlphaMap());
			}
			if (taskbar->showText)
			{
				taskbar->buttonfont->apply(destDC, buttonx, buttony, buttonwidth, buttonheight, caption, 0, 0);
			}
		}
	}
}

void TaskbarButton::OverlayButtonPainter(HDC hDC, int x, int y, int width, int height, int buttonType, int buttonType2)
{
	if ( !taskbar->ghostbuttonList.empty() )
	{
		for(GhostButtonListIterator glit = taskbar->ghostbuttonList.begin(); glit != taskbar->ghostbuttonList.end(); glit++)
		{
			if ( *glit != NULL )
			{
				int gbx = x + xParseCoord((*glit)->orgx, 0, width, taskbar->gethWnd());
				int gby =  y + xParseCoord((*glit)->orgy, 0, height, taskbar->gethWnd());
				int gbwidth = xParseDimen((*glit)->orgwidth, width, width, taskbar->gethWnd());
				int gbheight = xParseDimen((*glit)->orgheight, height, height, taskbar->gethWnd());

				(*glit)->buttontexture->applyOverlay(hDC, gbx, gby, gbwidth, gbheight, buttonType, buttonType2);
			}
			else
			{
				break;
			}
		}
	}
}
