/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "TaskbarTexture.h"
#include "math.h"

TaskbarTexture::TaskbarTexture() :
	m_Data(NULL),
	transparencyNeeded(false),
	useAlphaMap(false),
	memoryDC(0),
	memoryBitmap(0),
	useSolidColors(false)
{
}

TaskbarTexture::~TaskbarTexture()
{
	if ( m_Data != NULL )
		delete m_Data;

	Gradient.clear();

	if(memoryBitmap)
	{
		memoryBitmap = (HBITMAP) SelectObject(memoryDC, memoryBitmap);
		DeleteObject(memoryBitmap);
	}
	if(memoryDC)
		DeleteDC(memoryDC);
}

void TaskbarTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);

	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = GetSysColor(COLOR_3DLIGHT);
		darkcolor = GetSysColor(COLOR_3DSHADOW);
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void TaskbarTexture::parseBevelCFG(string cfg, int &leftBevel, int &rightBevel, int &topBevel, int &bottomBevel)
{
	char left[32], right[32], top[32], bottom[32];
	char *buffers[] = {left, right, top, bottom };

	leftBevel = 0;
	rightBevel = 0;
	topBevel = 0;
	bottomBevel = 0;

	int numTokens = LCTokenize(cfg.c_str(), buffers, 4, NULL);
		
	if(numTokens == 1)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = leftBevel;
		bottomBevel = leftBevel;
	}
	else if (numTokens == 2)
	{
		leftBevel = atoi(left);
		rightBevel = leftBevel;
		topBevel = atoi(right);
		bottomBevel = topBevel;
	}
	else if (numTokens == 4)
	{
		leftBevel = atoi(left);
		rightBevel = atoi(right);
		topBevel = atoi(top);
		bottomBevel = atoi(bottom);
	}
}

void TaskbarTexture::configure(const string &name, bool alphaMap)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	NameValuePair borderValues[] = 
	{
		{ ".none", 0 },
		{ "none", 0 },
		{ "raised", 1 },
		{ "sunken", 2 },
		{ "etched", 3 },
		{ "bump", 4 },
		{ 0, 0 }
	};

	NameValuePair gradientmodeConstants[] =
	{
		{ ".none", GRADIENT_NONE },
		{ ".horizontal", GRADIENT_HORIZONTAL },
		{ ".vertical", GRADIENT_VERTICAL },
		{ ".radial", GRADIENT_RADIAL },
		{ ".diagonal", GRADIENT_DIAGONAL },
		{ ".fdiagonal", GRADIENT_FDIAGONAL },
		{ ".bdiagonal", GRADIENT_BDIAGONAL },
		{ 0, 0 }
	};

	NameValuePair gradienttransformationConstants[] =
	{
		{ ".none", TRANSFORMATION_NONE },
		{ ".caricature", TRANSFORMATION_CHARICATURE },
		{ ".fisheye", TRANSFORMATION_FISHEYE },
		{ ".swirled", TRANSFORMATION_SWIRLED },
		{ ".cylinder", TRANSFORMATION_CYLINDER },
		{ ".shift", TRANSFORMATION_SHIFT },
		{ 0, 0 }
	};

	useAlphaMap = alphaMap;

	//Is useable in SolidColors and Image Mode
	borderMethod = GetxRCNamedValue(name, "BorderMethod", borderValues, 0);

	string imageFile = "";
	
	useSolidColors = GetxRCBool(name, "UseSolidColors");

	if ( useSolidColors )
	{
		imageFile = GetxRCLine(name, "SolidColors", "");
		if ( !imageFile.empty() )
		{
			parseCFG(imageFile, backgroundColor, lightBevelColor, darkBevelColor);
		}
		else
		{
			backgroundColor = GetSysColor(COLOR_BTNFACE);
			lightBevelColor = GetSysColor(COLOR_3DLIGHT);
			darkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		Gradient.insert(Gradient.end(), backgroundColor );

		if (backgroundColor == RGB(255,0,255) || lightBevelColor == RGB(255,0,255) || darkBevelColor == RGB(255,0,255) )
			transparencyNeeded = true; 

		imageFile = GetxRCLine(name, "SolidBevelSize", "0");
		parseBevelCFG(imageFile, bevelSizeLeft, bevelSizeRight, bevelSizeTop, bevelSizeBottom);

		verticalGradient = GetxRCBool(name, "SolidGradientVertical");
		repeatedGradient = GetxRCInt(name, "SolidGradientRepeated", 0);

		//Win2K/XP Special Gradient Modes
		gradientType = GetxRCNamedValue(name, "SolidGradientType", gradientmodeConstants, -1);
		transformationType = GetxRCNamedValue(name, "SolidTransformationType", gradienttransformationConstants, -1);

		imageFile = GetxRCLine(name, "SolidGradientColors", "");
	
		if ( !imageFile.empty() )
		{	
			int color;
			COLORREF insertColor;
			char token[4096];
			LPCSTR nextToken = imageFile.c_str();

			while (GetToken(nextToken, token, &nextToken, false))
			{
				color = strtol(token, NULL, 16);
				insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		
				Gradient.insert(Gradient.end(), insertColor );
			}
		}
	}
	else
	{
		imageFile = GetxRCString(name, "Image", "");
		memoryBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memoryBitmap)
		{
			GetLSBitmapSize(memoryBitmap, &imageWidth, &imageHeight);

			memoryDC = CreateCompatibleDC(NULL);
			SelectObject(memoryDC, memoryBitmap);

			taskbar_hueColor = GetxRCColor(name, "HueColor", RGB(255, 255, 255));
			taskbar_hueIntensity = GetxRCInt(name, "HueIntensity", 0, 0, 100);
			taskbar_luminanceIntensity = GetxRCInt(name, "LuminanceIntensity", 0, -100, 100);
			taskbar_saturationIntensity = GetxRCInt(name, "SaturationIntensity", 100, 0, 200);

			taskbar_mixColor = GetxRCColor(name, "MixColor", RGB(255, 255, 255));
			taskbar_mixIntensity = GetxRCInt(name, "MixIntensity", 0, 0, 100);
	
			leftEdge = GetxRCInt(name, "ImageLeftEdge", 0, 0);
			topEdge = GetxRCInt(name, "ImageTopEdge", 0, 0);
			rightEdge = GetxRCInt(name, "ImageRightEdge", 0, 0);
			bottomEdge = GetxRCInt(name, "ImageBottomEdge", 0, 0);

			mode = GetxRCNamedValue(name, "ImageMode", modeConstants, MULTIBLT_STRETCH);

			if (useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryBitmap);
			}
			else
			{
				paintEffects(memoryDC, imageWidth, imageHeight);
			}
		}
		else
		{
			transparencyNeeded = true;
		}
	}
}

void TaskbarTexture::apply(HDC hDC, int x, int y, int width, int height, bool trueTransparency)
{
	if ( width <= 0 || height <= 0 )
		return;

	if (useSolidColors)
	{
		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

		RECT r;
		r.left = 0;
		r.top = 0;
		r.right = width;
		r.bottom = height;

		int bevelWidth = bevelSizeLeft+bevelSizeRight;
		int bevelHeight = bevelSizeTop+bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bevelSizeRight;
			r.bottom = r.bottom-bevelSizeBottom;
			::SetBkColor(bufferDC, lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bevelSizeLeft;
			r.top = r.top+bevelSizeTop;

			if ( Gradient.size() > 1 )
			{
				if ( gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, r);
				else
					DrawGradient(bufferDC, r, Gradient[0], Gradient[1], gradientType, transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( Gradient.size() > 1 )
			{
				if ( gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, r);
				else
					DrawGradient(bufferDC, r, Gradient[0], Gradient[1], gradientType, transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}

		if ( trueTransparency )
		{
			BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
		}
		else
		{
			TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}
	else
	{
		if (memoryBitmap == 0)
			return;

		if (trueTransparency)
		{
			SetStretchBltMode(hDC, STRETCH_DELETESCANS);

			MultiBlt(hDC, x, y, width, height,
						memoryDC, 0, 0, imageWidth, imageHeight,
						leftEdge, topEdge, rightEdge, bottomEdge,
						mode, SRCCOPY);
		}
		else if (!trueTransparency)
		{
			HDC bufferDC = CreateCompatibleDC(hDC);
			HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
			HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

			SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryDC, 0, 0, imageWidth, imageHeight,
					leftEdge, topEdge, rightEdge, bottomEdge,
					mode, SRCCOPY);
				
			TransparentBltLS(hDC, x, y, width, height,
					bufferDC, 0, 0,
					RGB(255, 0, 255));

			bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
			DeleteObject(bufferBitmap);
			DeleteDC(bufferDC);
		}
	}

	RECT completeFillRect = {x, y, x+width, y+height};

	if (borderMethod == 1)
		DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (borderMethod == 2)
		DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (borderMethod == 3)
		DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (borderMethod == 4)
		DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
}

void TaskbarTexture::applyOverlay(HDC hDC, int x, int y, int width, int height, int alphaValue)
{
	if ( width <= 0 || height <= 0 )
		return;

	HDC bufferDC = CreateCompatibleDC(NULL);

	BITMAPINFO alpha;
	ZeroMemory( &alpha.bmiHeader, sizeof(BITMAPINFOHEADER) );
	alpha.bmiHeader.biWidth=width;      // Set size you need
	alpha.bmiHeader.biHeight=height;    // Set size you need
	alpha.bmiHeader.biPlanes=1;
	alpha.bmiHeader.biBitCount=32;      // Can be 8, 16, 32 bpp or other number
	alpha.bmiHeader.biSizeImage=0;
	alpha.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	alpha.bmiHeader.biClrUsed= 0;
	alpha.bmiHeader.biClrImportant= 0;
	VOID *pvBits;

	HBITMAP bufferBitmap = CreateDIBSection( bufferDC,
												&alpha,
												DIB_RGB_COLORS,
												&pvBits,
												NULL,
												0 );

	HBITMAP oldBMP = (HBITMAP) SelectObject(bufferDC, bufferBitmap);

	SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

	if (useSolidColors)
	{
		RECT r;
		r.left = 0;
		r.top = 0;
		r.right = width;
		r.bottom = height;

		int bevelWidth = bevelSizeLeft+bevelSizeRight;
		int bevelHeight = bevelSizeTop+bevelSizeBottom;

		// fill in the background
		if (bevelWidth+bevelHeight > 0 && (bevelWidth < width && bevelHeight < height))
		{
			::SetBkColor(bufferDC, darkBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-bevelSizeRight;
			r.bottom = r.bottom-bevelSizeBottom;
			::SetBkColor(bufferDC, lightBevelColor);
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+bevelSizeLeft;
			r.top = r.top+bevelSizeTop;

			if ( Gradient.size() > 1 )
			{
				if ( gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, r);
				else
					DrawGradient(hDC, r, Gradient[0], Gradient[1], gradientType, transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( Gradient.size() > 1 )
			{
				if ( gradientType == -1 || !IsOS(OS_2KXP) )
					solidGradientPaint(bufferDC, r);
				else
					DrawGradient(hDC, r, Gradient[0], Gradient[1], gradientType, transformationType);
			}
			else
			{
				::SetBkColor(bufferDC, backgroundColor);
				::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if (memoryBitmap == 0)
			return;

		MultiBlt(bufferDC, 0, 0, width, height,
						memoryDC, 0, 0, imageWidth, imageHeight,
						leftEdge, topEdge, rightEdge, bottomEdge,
						mode, SRCCOPY);
	}
	
	RECT completeFillRect = {0, 0, width, height};

	if (borderMethod == 1)
		DrawEdge(bufferDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
	else if (borderMethod == 2)
		DrawEdge(bufferDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
	else if (borderMethod == 3)
		DrawEdge(bufferDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
	else if (borderMethod == 4)
		DrawEdge(bufferDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);

	BITMAP bm;
	GetObject(bufferBitmap, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int j = 0; j < iSize; ++j)
	{
		//Remove Magic Pink
		if ( (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255) )
		{
			pPixel[0] = 0;
			pPixel[1] = 0;
			pPixel[2] = 0;
			pPixel[3] = 0;
		}
		else if ( !useAlphaMap || pPixel[3] == 0 || useSolidColors ) 
		{
			pPixel[3] = 255;
		}

		pPixel+= 4;
	}
	
	BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, alphaValue, AC_SRC_ALPHA };

	AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);

	bufferBitmap = (HBITMAP) SelectObject(bufferDC, oldBMP);
	DeleteObject(bufferBitmap);
	DeleteDC(bufferDC);
}

void TaskbarTexture::paintEffects(HDC destHDC, int width, int height)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;
	
	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( taskbar_luminanceIntensity != 0 || taskbar_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( taskbar_hueIntensity != 0 )
	{
		r = GetRValue(taskbar_hueColor);
		g = GetGValue(taskbar_hueColor);
		b = GetBValue(taskbar_hueColor);

		if ( r == b && b == g )
		{
			taskbar_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(r, g, b, hueh, hues, huel);
			needHSLConversion = true;
		}
	}

	if ( taskbar_mixIntensity != 0 )
	{
		taskbar_mixIntensity = (int) min(255, taskbar_mixIntensity*2.55);
		needMixColor = true;
	}

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
			
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				if ( needHSLConversion )
				{
					RGB2HSL(r, g, b, h, s, l);

					if(taskbar_hueIntensity != 0)
						h = (BYTE) max(0, min(240, (h*(100-taskbar_hueIntensity)+hueh*taskbar_hueIntensity)/100));
		
					if(taskbar_luminanceIntensity != 0)
						l = (BYTE) max(0, min(240, ((l*100)+(120*taskbar_luminanceIntensity))/100));

					if(taskbar_saturationIntensity != 100)
					{
						if ( taskbar_saturationIntensity > 100 )
							s = (BYTE) max(0, min(240, (240*(taskbar_saturationIntensity-100))/100));				
						else
							s = (BYTE) max(0, min(240, (s*taskbar_saturationIntensity)/100));
					}
				
					HSL2RGB(r, g, b, h, s, l);
				}

				if ( needMixColor )
				{
					// HueMix effect
					r = (BYTE)((r * (255 - taskbar_mixIntensity) + GetRValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - taskbar_mixIntensity) + GetGValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - taskbar_mixIntensity) + GetBValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
				}
			}
			else
			{
				transparencyNeeded = true;
				if ( !needHSLConversion && !needMixColor )
					return;
			}

			if ( needHSLConversion || needMixColor )
				SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void TaskbarTexture::solidGradientPaint(HDC destHDC, RECT r)
{
	if ( repeatedGradient == 0 )
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		double cr, cg, cb, dr, dg, db;

		// ColorsSteps between Start and End Color
		int gradSteps = 0;
		int unequalRest = 0;

		int colorCount = Gradient.size();

		if (verticalGradient)
		{
			gradSteps = (nHeight-colorCount)/(colorCount-1);
			unequalRest = (nHeight-colorCount)%(colorCount-1);
		}
		else
		{
			gradSteps = (nWidth-colorCount)/(colorCount-1);
			unequalRest = (nWidth-colorCount)%(colorCount-1);
		}

		int lastGradientPos = 0;
		if (verticalGradient)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		for (ColorRefListIterator cit = Gradient.begin(); cit != Gradient.end(); cit++ )
		{
			if (cit == Gradient.end())
				break;

			counter++;

			COLORREF one = *cit;
			cit++;
			COLORREF two = *cit;
			cit--;
		
			// get current color
			cr = GetRValue( one );
			cg = GetGValue( one );
			cb = GetBValue( one );

			// Get delta color (+1 cause we must reach the Final Color)
			dr = (cr - GetRValue( two )) / (gradSteps+1);
			dg = (cg - GetGValue( two )) / (gradSteps+1);
			db = (cb - GetBValue( two )) / (gradSteps+1);

			//Paint Real Gradient Color
			//-------------------------------------------
			if ( unequalRest != 0 && counter > 1 )
			{
				unequalRest--;
				if (verticalGradient)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, one);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				if (verticalGradient)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;
			}
			//-------------------------------------------

			for (int i = 0; i <= gradSteps; i++)
			{
				if (verticalGradient)
				{
					SetRect(&r2, r.left, lastGradientPos+i, r.right, min(lastGradientPos+i+1, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos+i, r.top, min(lastGradientPos+i+1, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}
				
				cr -= dr;
				cg -= dg;
				cb -= db;
			}

			if (verticalGradient)
				lastGradientPos = r2.bottom;
			else
				lastGradientPos = r2.right;
		}
	}
	else
	{
		int nWidth = r.right-r.left;
		int nHeight = r.bottom-r.top;

		RECT r2 = {0,0,0,0};
		COLORREF color;

		int lastGradientPos = 0;
		if (verticalGradient)
			lastGradientPos = r.top;
		else
			lastGradientPos = r.left;

		int counter = 0;
		int nSize = 0;
		
		if (verticalGradient)
			nSize = nHeight;
		else
			nSize = nWidth; 

		while ( counter < nSize )
		{
			for (ColorRefListIterator cit = Gradient.begin(); cit != Gradient.end(); cit++ )
			{
				color = *cit;

				if (verticalGradient)
				{
					SetRect(&r2, r.left, lastGradientPos, r.right, min(lastGradientPos+repeatedGradient, r.bottom));
				}
				else
				{
					SetRect(&r2, lastGradientPos, r.top, min(lastGradientPos+repeatedGradient, r.right), r.bottom);
				}

				if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
				{
					::SetBkColor(destHDC, color);
					::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
				}

				if (verticalGradient)
					lastGradientPos = r2.bottom;
				else
					lastGradientPos = r2.right;

				counter++;

				if ( counter == nSize )
					break;
			}
		}
	}
}

void TaskbarTexture::DrawGradient( HDC hDC, RECT& rect, COLORREF startColor, COLORREF endColor, int gradientType, int transformationType)
{
	int i, j, y, x1, y1;
	double r, d, n1;

	// Calculate image size
	int width = rect.right - rect.left;
	int height = rect.bottom - rect.top;
	int size = height*width*4;
	double radius = sqrt( pow(height/2,2) + pow(width/2,2) );
	double k = double(rect.bottom-rect.top)/double(rect.right-rect.left);
	if ( gradientType == GRADIENT_BDIAGONAL )
		k = double(rect.bottom-rect.top)/double(rect.left-rect.right);
	if ( ( gradientType == GRADIENT_FDIAGONAL ) || ( gradientType == GRADIENT_BDIAGONAL ) )
		gradientType = GRADIENT_DIAGONAL;
	double n = 0.0;
	double k1 = -1/k;

	// Create data buffer
	m_Data = new unsigned char[size];
	
	// Create gradient
	for ( i=0; i<height; i++ )
	{
		for ( j=0; j<width; j++ )
		{
			switch (gradientType)
			{
				// Horizontal gradient
				case GRADIENT_HORIZONTAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)j/(double)width)*GetBValue(endColor) + (1-(double)j/(double)width)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)j/(double)width)*GetGValue(endColor) + (1-(double)j/(double)width)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)j/(double)width)*GetRValue(endColor) + (1-(double)j/(double)width)*GetRValue(startColor));
				}
				break;

				// Vertical gradient
				case GRADIENT_VERTICAL:
				{
					m_Data[i*width*4 + j*4] = (unsigned char)(((double)i/(double)height)*GetBValue(endColor) + (1-(double)i/(double)height)*GetBValue(startColor));
					m_Data[i*width*4 + j*4+1] = (unsigned char)(((double)i/(double)height)*GetGValue(endColor) + (1-(double)i/(double)height)*GetGValue(startColor));
					m_Data[i*width*4 + j*4+2] = (unsigned char)(((double)i/(double)height)*GetRValue(endColor) + (1-(double)i/(double)height)*GetRValue(startColor));
				}
				break;

				// Radial gradient
				case GRADIENT_RADIAL:
				{
					r = sqrt( pow(double(i-height/2)/radius,2) + pow(double(j-width/2)/radius,2) );
					if ( r <= 1.0 )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)(r*GetBValue(endColor) + (1-r)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)(r*GetGValue(endColor) + (1-r)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)(r*GetRValue(endColor) + (1-r)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = GetBValue(endColor);
						m_Data[i*width*4 + j*4+1] = GetGValue(endColor);
						m_Data[i*width*4 + j*4+2] = GetRValue(endColor);
					}
				}
				break;

				// Diagonal gradient
				case GRADIENT_DIAGONAL:
				{
					y = int(k*(j-width/2) + n);
					n1 = int((i-height/2) - k1*(j-width/2));
					x1 = int((n1-n)/(k-k1));
					y1 = int(k1*x1 + n1);
					d = sqrt( pow((j-width/2)-x1,2) + pow((i-height/2)-y1,2) ) / (2*radius);
					if ( (i-height/2) > y )
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(endColor) + (0.5+d)*GetBValue(startColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(endColor) + (0.5+d)*GetGValue(startColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(endColor) + (0.5+d)*GetRValue(startColor));
					}
					else
					{
						m_Data[i*width*4 + j*4] = (unsigned char)((0.5-d)*GetBValue(startColor) + (0.5+d)*GetBValue(endColor));
						m_Data[i*width*4 + j*4+1] = (unsigned char)((0.5-d)*GetGValue(startColor) + (0.5+d)*GetGValue(endColor));
						m_Data[i*width*4 + j*4+2] = (unsigned char)((0.5-d)*GetRValue(startColor) + (0.5+d)*GetRValue(endColor));
					}
				}
				break;
			}
		}
	}

	if ( transformationType != TRANSFORMATION_NONE )
		ApplyTransformation( width, height, transformationType );

	// Create temporary bitmap
	HBITMAP hBitmap = (HBITMAP)CreateBitmap( width, height, 1, 32, m_Data );

	// Create temporary device context
	HDC hMemDC = CreateCompatibleDC(hDC);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// Blit gradient bitmap
	BitBlt( hDC, rect.left, rect.top, width, height, hMemDC, 0, 0, SRCCOPY );

	// Free memory
	SelectObject( hMemDC, hOldBitmap );
	DeleteDC(hMemDC);
	DeleteObject(hBitmap);
	delete m_Data;
	m_Data = NULL;
}

void TaskbarTexture::ApplyTransformation(int width, int height, int transformationType)
{
	int X, Y;
	double r, fi;
	double R = sqrt( pow(width/4,2) + pow(height/4,2) );
	double FI = atan( double(height)/double(width) );

	int size = height*width*4;
	unsigned char* newBuffer = new unsigned char[size];
	memcpy( newBuffer, m_Data, size );

	for ( int i=0; i<height; i++ )
	{
		for ( int j=0; j<width; j++ )
		{
			switch (transformationType)
			{
				case TRANSFORMATION_CHARICATURE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(sqrt(r*R*2)*cos(fi));
					Y = int(fabs(sqrt(r*R*2)*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_FISHEYE:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int((pow(r,2)/R)*cos(fi));
					Y = int(fabs((pow(r,2)/R)*sin(fi))) % height;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SWIRLED:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi+r/8)));
					Y = int(fabs(r*sin(fi+r/8)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_CYLINDER:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = abs((j-width/2) + int(fi+r/10)%32 - 8);
					Y = i;
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;

				case TRANSFORMATION_SHIFT:
				{
					fi = atan(double(i-height/2)/double(j-width/2));
					r = sqrt( pow((i-height/2),2) + pow((j-width/2),2) );
					X = int(fabs(r*cos(fi)));
					Y = int(fabs(r*sin(fi)));
					newBuffer[i*width*4 + j*4] = m_Data[Y*width*4 + X*4];
					newBuffer[i*width*4 + j*4+1] = m_Data[Y*width*4 + X*4+1];
					newBuffer[i*width*4 + j*4+2] = m_Data[Y*width*4 + X*4+2];
				}
				break;
			}
		}
	}

	memcpy( m_Data, newBuffer, size );
	delete newBuffer;
}

void TaskbarTexture::PrepareAlphaChannel(HBITMAP &bmp)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	BYTE h = 0;
	BYTE s = 0;
	BYTE l = 0;
	
	BYTE hueh = 0;
	BYTE hues = 0;
	BYTE huel = 0;

	bool needHSLConversion = false;
	bool needMixColor = false;

	if ( taskbar_luminanceIntensity != 0 || taskbar_saturationIntensity != 100 )
		needHSLConversion = true;

	if ( taskbar_hueIntensity != 0 )
	{
		r = GetRValue(taskbar_hueColor);
		g = GetGValue(taskbar_hueColor);
		b = GetBValue(taskbar_hueColor);

		if ( r == b && b == g )
		{
			taskbar_hueIntensity = 0;
		}
		else
		{
			RGB2HSL(b, g, r, hueh, hues, huel); //Switch r and b due to AlphaMap/bm_b.bmHeight * bm_b.bmWidth
			needHSLConversion = true;
		}
	}

	if ( taskbar_mixIntensity != 0 )
	{
		taskbar_mixIntensity = (int) min(255, taskbar_mixIntensity*2.55);
		needMixColor = true;
	}

	int iSize = bm.bmHeight * bm.bmWidth;

	for (int i = 0; i < iSize; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		if ( needHSLConversion )
		{
			//AlphaMap Colorizing
			RGB2HSL(r, g, b, h, s, l);

			if(taskbar_hueIntensity != 0)
				h = (BYTE) max(0, min(240, (h*(100-taskbar_hueIntensity)+hueh*taskbar_hueIntensity)/100));
		
			if(taskbar_luminanceIntensity != 0)
				l = (BYTE) max(0, min(240, ((l*100)+(120*taskbar_luminanceIntensity))/100));

			if(taskbar_saturationIntensity != 100)
			{
				if ( taskbar_saturationIntensity > 100 )
					s = (BYTE) max(0, min(240, (240*(taskbar_saturationIntensity-100))/100));				
				else
					s = (BYTE) max(0, min(240, (s*taskbar_saturationIntensity)/100));
			}

			HSL2RGB(r, g, b, h, s, l);
		}

		if ( needMixColor )
		{
			// HueMix effect
			r = (BYTE)((r * (255 - taskbar_mixIntensity) + GetBValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - taskbar_mixIntensity) + GetGValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
			b = (BYTE)((b * (255 - taskbar_mixIntensity) + GetRValue(taskbar_mixColor) * taskbar_mixIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}